from __future__ import annotations

# pylint: disable=too-many-lines
types: list[str] = [
    "AWS::ACMPCA::Certificate",
    "AWS::ACMPCA::CertificateAuthority",
    "AWS::ACMPCA::CertificateAuthorityActivation",
    "AWS::ACMPCA::Permission",
    "AWS::ARCZonalShift::AutoshiftObserverNotificationStatus",
    "AWS::ARCZonalShift::ZonalAutoshiftConfiguration",
    "AWS::AccessAnalyzer::Analyzer",
    "AWS::AmazonMQ::Broker",
    "AWS::AmazonMQ::Configuration",
    "AWS::AmazonMQ::ConfigurationAssociation",
    "AWS::ApiGateway::Account",
    "AWS::ApiGateway::ApiKey",
    "AWS::ApiGateway::Authorizer",
    "AWS::ApiGateway::BasePathMapping",
    "AWS::ApiGateway::ClientCertificate",
    "AWS::ApiGateway::Deployment",
    "AWS::ApiGateway::DocumentationPart",
    "AWS::ApiGateway::DocumentationVersion",
    "AWS::ApiGateway::DomainName",
    "AWS::ApiGateway::GatewayResponse",
    "AWS::ApiGateway::Method",
    "AWS::ApiGateway::Model",
    "AWS::ApiGateway::RequestValidator",
    "AWS::ApiGateway::Resource",
    "AWS::ApiGateway::RestApi",
    "AWS::ApiGateway::Stage",
    "AWS::ApiGateway::UsagePlan",
    "AWS::ApiGateway::UsagePlanKey",
    "AWS::ApiGateway::VpcLink",
    "AWS::ApiGatewayV2::Api",
    "AWS::ApiGatewayV2::ApiMapping",
    "AWS::ApiGatewayV2::Authorizer",
    "AWS::ApiGatewayV2::Deployment",
    "AWS::ApiGatewayV2::DomainName",
    "AWS::ApiGatewayV2::Integration",
    "AWS::ApiGatewayV2::IntegrationResponse",
    "AWS::ApiGatewayV2::Model",
    "AWS::ApiGatewayV2::Route",
    "AWS::ApiGatewayV2::RouteResponse",
    "AWS::ApiGatewayV2::Stage",
    "AWS::AppConfig::Application",
    "AWS::AppConfig::ConfigurationProfile",
    "AWS::AppConfig::Deployment",
    "AWS::AppConfig::DeploymentStrategy",
    "AWS::AppConfig::Environment",
    "AWS::AppConfig::Extension",
    "AWS::AppConfig::ExtensionAssociation",
    "AWS::AppConfig::HostedConfigurationVersion",
    "AWS::AppMesh::GatewayRoute",
    "AWS::AppMesh::Mesh",
    "AWS::AppMesh::Route",
    "AWS::AppMesh::VirtualGateway",
    "AWS::AppMesh::VirtualNode",
    "AWS::AppMesh::VirtualRouter",
    "AWS::AppMesh::VirtualService",
    "AWS::AppSync::ApiCache",
    "AWS::AppSync::ApiKey",
    "AWS::AppSync::DataSource",
    "AWS::AppSync::DomainName",
    "AWS::AppSync::DomainNameApiAssociation",
    "AWS::AppSync::FunctionConfiguration",
    "AWS::AppSync::GraphQLApi",
    "AWS::AppSync::GraphQLSchema",
    "AWS::AppSync::Resolver",
    "AWS::AppSync::SourceApiAssociation",
    "AWS::ApplicationAutoScaling::ScalableTarget",
    "AWS::ApplicationAutoScaling::ScalingPolicy",
    "AWS::ApplicationSignals::ServiceLevelObjective",
    "AWS::Athena::DataCatalog",
    "AWS::Athena::NamedQuery",
    "AWS::Athena::PreparedStatement",
    "AWS::Athena::WorkGroup",
    "AWS::AutoScaling::AutoScalingGroup",
    "AWS::AutoScaling::LaunchConfiguration",
    "AWS::AutoScaling::LifecycleHook",
    "AWS::AutoScaling::ScalingPolicy",
    "AWS::AutoScaling::ScheduledAction",
    "AWS::AutoScaling::WarmPool",
    "AWS::Backup::BackupPlan",
    "AWS::Backup::BackupSelection",
    "AWS::Backup::BackupVault",
    "AWS::Batch::ComputeEnvironment",
    "AWS::Batch::JobDefinition",
    "AWS::Batch::JobQueue",
    "AWS::Batch::SchedulingPolicy",
    "AWS::CDK::Metadata",
    "AWS::CE::AnomalyMonitor",
    "AWS::CE::AnomalySubscription",
    "AWS::CertificateManager::Certificate",
    "AWS::Chatbot::MicrosoftTeamsChannelConfiguration",
    "AWS::Chatbot::SlackChannelConfiguration",
    "AWS::Cloud9::EnvironmentEC2",
    "AWS::CloudFormation::CustomResource",
    "AWS::CloudFormation::HookDefaultVersion",
    "AWS::CloudFormation::HookTypeConfig",
    "AWS::CloudFormation::HookVersion",
    "AWS::CloudFormation::Macro",
    "AWS::CloudFormation::PublicTypeVersion",
    "AWS::CloudFormation::ResourceDefaultVersion",
    "AWS::CloudFormation::ResourceVersion",
    "AWS::CloudFormation::Stack",
    "AWS::CloudFormation::StackSet",
    "AWS::CloudFormation::WaitCondition",
    "AWS::CloudFormation::WaitConditionHandle",
    "AWS::CloudFront::CachePolicy",
    "AWS::CloudFront::CloudFrontOriginAccessIdentity",
    "AWS::CloudFront::ContinuousDeploymentPolicy",
    "AWS::CloudFront::Distribution",
    "AWS::CloudFront::Function",
    "AWS::CloudFront::KeyGroup",
    "AWS::CloudFront::KeyValueStore",
    "AWS::CloudFront::MonitoringSubscription",
    "AWS::CloudFront::OriginAccessControl",
    "AWS::CloudFront::OriginRequestPolicy",
    "AWS::CloudFront::PublicKey",
    "AWS::CloudFront::RealtimeLogConfig",
    "AWS::CloudFront::ResponseHeadersPolicy",
    "AWS::CloudTrail::EventDataStore",
    "AWS::CloudTrail::Trail",
    "AWS::CloudWatch::Alarm",
    "AWS::CloudWatch::AnomalyDetector",
    "AWS::CloudWatch::CompositeAlarm",
    "AWS::CloudWatch::Dashboard",
    "AWS::CloudWatch::InsightRule",
    "AWS::CloudWatch::MetricStream",
    "AWS::CodeBuild::Project",
    "AWS::CodeBuild::ReportGroup",
    "AWS::CodeBuild::SourceCredential",
    "AWS::CodeDeploy::Application",
    "AWS::CodeDeploy::DeploymentConfig",
    "AWS::CodeDeploy::DeploymentGroup",
    "AWS::CodePipeline::CustomActionType",
    "AWS::CodePipeline::Pipeline",
    "AWS::Cognito::IdentityPool",
    "AWS::Cognito::IdentityPoolPrincipalTag",
    "AWS::Cognito::IdentityPoolRoleAttachment",
    "AWS::Cognito::LogDeliveryConfiguration",
    "AWS::Cognito::UserPool",
    "AWS::Cognito::UserPoolClient",
    "AWS::Cognito::UserPoolDomain",
    "AWS::Cognito::UserPoolGroup",
    "AWS::Cognito::UserPoolIdentityProvider",
    "AWS::Cognito::UserPoolResourceServer",
    "AWS::Cognito::UserPoolRiskConfigurationAttachment",
    "AWS::Cognito::UserPoolUICustomizationAttachment",
    "AWS::Cognito::UserPoolUser",
    "AWS::Cognito::UserPoolUserToGroupAttachment",
    "AWS::Config::AggregationAuthorization",
    "AWS::Config::ConfigRule",
    "AWS::Config::ConfigurationAggregator",
    "AWS::Config::ConfigurationRecorder",
    "AWS::Config::DeliveryChannel",
    "AWS::Config::OrganizationConfigRule",
    "AWS::Config::RemediationConfiguration",
    "AWS::Config::StoredQuery",
    "AWS::ControlTower::EnabledBaseline",
    "AWS::ControlTower::EnabledControl",
    "AWS::ControlTower::LandingZone",
    "AWS::DLM::LifecyclePolicy",
    "AWS::DMS::DataProvider",
    "AWS::DMS::InstanceProfile",
    "AWS::DMS::MigrationProject",
    "AWS::DMS::ReplicationConfig",
    "AWS::DataPipeline::Pipeline",
    "AWS::DataSync::Agent",
    "AWS::DataSync::LocationAzureBlob",
    "AWS::DataSync::LocationEFS",
    "AWS::DataSync::LocationFSxLustre",
    "AWS::DataSync::LocationFSxONTAP",
    "AWS::DataSync::LocationFSxWindows",
    "AWS::DataSync::LocationHDFS",
    "AWS::DataSync::LocationNFS",
    "AWS::DataSync::LocationObjectStorage",
    "AWS::DataSync::LocationS3",
    "AWS::DataSync::LocationSMB",
    "AWS::DataSync::Task",
    "AWS::Detective::Graph",
    "AWS::Detective::MemberInvitation",
    "AWS::Detective::OrganizationAdmin",
    "AWS::DirectoryService::MicrosoftAD",
    "AWS::DirectoryService::SimpleAD",
    "AWS::DynamoDB::GlobalTable",
    "AWS::DynamoDB::Table",
    "AWS::EC2::CapacityReservation",
    "AWS::EC2::CapacityReservationFleet",
    "AWS::EC2::ClientVpnAuthorizationRule",
    "AWS::EC2::ClientVpnEndpoint",
    "AWS::EC2::ClientVpnRoute",
    "AWS::EC2::ClientVpnTargetNetworkAssociation",
    "AWS::EC2::CustomerGateway",
    "AWS::EC2::DHCPOptions",
    "AWS::EC2::EC2Fleet",
    "AWS::EC2::EIP",
    "AWS::EC2::EIPAssociation",
    "AWS::EC2::EgressOnlyInternetGateway",
    "AWS::EC2::FlowLog",
    "AWS::EC2::GatewayRouteTableAssociation",
    "AWS::EC2::Host",
    "AWS::EC2::IPAM",
    "AWS::EC2::IPAMAllocation",
    "AWS::EC2::IPAMPool",
    "AWS::EC2::IPAMPoolCidr",
    "AWS::EC2::IPAMResourceDiscovery",
    "AWS::EC2::IPAMResourceDiscoveryAssociation",
    "AWS::EC2::IPAMScope",
    "AWS::EC2::Instance",
    "AWS::EC2::InstanceConnectEndpoint",
    "AWS::EC2::InternetGateway",
    "AWS::EC2::KeyPair",
    "AWS::EC2::LaunchTemplate",
    "AWS::EC2::LocalGatewayRoute",
    "AWS::EC2::LocalGatewayRouteTable",
    "AWS::EC2::LocalGatewayRouteTableVPCAssociation",
    "AWS::EC2::LocalGatewayRouteTableVirtualInterfaceGroupAssociation",
    "AWS::EC2::NatGateway",
    "AWS::EC2::NetworkAcl",
    "AWS::EC2::NetworkAclEntry",
    "AWS::EC2::NetworkInterface",
    "AWS::EC2::NetworkInterfaceAttachment",
    "AWS::EC2::NetworkInterfacePermission",
    "AWS::EC2::PlacementGroup",
    "AWS::EC2::PrefixList",
    "AWS::EC2::Route",
    "AWS::EC2::RouteTable",
    "AWS::EC2::SecurityGroup",
    "AWS::EC2::SecurityGroupEgress",
    "AWS::EC2::SecurityGroupIngress",
    "AWS::EC2::SnapshotBlockPublicAccess",
    "AWS::EC2::SpotFleet",
    "AWS::EC2::Subnet",
    "AWS::EC2::SubnetCidrBlock",
    "AWS::EC2::SubnetNetworkAclAssociation",
    "AWS::EC2::SubnetRouteTableAssociation",
    "AWS::EC2::TrafficMirrorFilter",
    "AWS::EC2::TrafficMirrorFilterRule",
    "AWS::EC2::TrafficMirrorSession",
    "AWS::EC2::TrafficMirrorTarget",
    "AWS::EC2::TransitGateway",
    "AWS::EC2::TransitGatewayAttachment",
    "AWS::EC2::TransitGatewayConnect",
    "AWS::EC2::TransitGatewayMulticastDomain",
    "AWS::EC2::TransitGatewayMulticastDomainAssociation",
    "AWS::EC2::TransitGatewayMulticastGroupMember",
    "AWS::EC2::TransitGatewayMulticastGroupSource",
    "AWS::EC2::TransitGatewayPeeringAttachment",
    "AWS::EC2::TransitGatewayRoute",
    "AWS::EC2::TransitGatewayRouteTable",
    "AWS::EC2::TransitGatewayRouteTableAssociation",
    "AWS::EC2::TransitGatewayRouteTablePropagation",
    "AWS::EC2::TransitGatewayVpcAttachment",
    "AWS::EC2::VPC",
    "AWS::EC2::VPCCidrBlock",
    "AWS::EC2::VPCDHCPOptionsAssociation",
    "AWS::EC2::VPCEndpoint",
    "AWS::EC2::VPCEndpointConnectionNotification",
    "AWS::EC2::VPCEndpointService",
    "AWS::EC2::VPCEndpointServicePermissions",
    "AWS::EC2::VPCGatewayAttachment",
    "AWS::EC2::VPCPeeringConnection",
    "AWS::EC2::VPNConnection",
    "AWS::EC2::VPNConnectionRoute",
    "AWS::EC2::VPNGateway",
    "AWS::EC2::VPNGatewayRoutePropagation",
    "AWS::EC2::VerifiedAccessEndpoint",
    "AWS::EC2::VerifiedAccessGroup",
    "AWS::EC2::VerifiedAccessInstance",
    "AWS::EC2::VerifiedAccessTrustProvider",
    "AWS::EC2::Volume",
    "AWS::EC2::VolumeAttachment",
    "AWS::ECR::PullThroughCacheRule",
    "AWS::ECR::RegistryPolicy",
    "AWS::ECR::ReplicationConfiguration",
    "AWS::ECR::Repository",
    "AWS::ECR::RepositoryCreationTemplate",
    "AWS::ECS::CapacityProvider",
    "AWS::ECS::Cluster",
    "AWS::ECS::ClusterCapacityProviderAssociations",
    "AWS::ECS::PrimaryTaskSet",
    "AWS::ECS::Service",
    "AWS::ECS::TaskDefinition",
    "AWS::ECS::TaskSet",
    "AWS::EFS::AccessPoint",
    "AWS::EFS::FileSystem",
    "AWS::EFS::MountTarget",
    "AWS::EKS::AccessEntry",
    "AWS::EKS::Addon",
    "AWS::EKS::Cluster",
    "AWS::EKS::FargateProfile",
    "AWS::EKS::IdentityProviderConfig",
    "AWS::EKS::Nodegroup",
    "AWS::EKS::PodIdentityAssociation",
    "AWS::EMR::Cluster",
    "AWS::EMR::InstanceFleetConfig",
    "AWS::EMR::InstanceGroupConfig",
    "AWS::EMR::SecurityConfiguration",
    "AWS::EMR::Step",
    "AWS::EMR::Studio",
    "AWS::EMR::StudioSessionMapping",
    "AWS::ElastiCache::CacheCluster",
    "AWS::ElastiCache::ParameterGroup",
    "AWS::ElastiCache::ReplicationGroup",
    "AWS::ElastiCache::SecurityGroup",
    "AWS::ElastiCache::SecurityGroupIngress",
    "AWS::ElastiCache::ServerlessCache",
    "AWS::ElastiCache::SubnetGroup",
    "AWS::ElastiCache::User",
    "AWS::ElastiCache::UserGroup",
    "AWS::ElasticBeanstalk::Application",
    "AWS::ElasticBeanstalk::ApplicationVersion",
    "AWS::ElasticBeanstalk::ConfigurationTemplate",
    "AWS::ElasticBeanstalk::Environment",
    "AWS::ElasticLoadBalancing::LoadBalancer",
    "AWS::ElasticLoadBalancingV2::Listener",
    "AWS::ElasticLoadBalancingV2::ListenerCertificate",
    "AWS::ElasticLoadBalancingV2::ListenerRule",
    "AWS::ElasticLoadBalancingV2::LoadBalancer",
    "AWS::ElasticLoadBalancingV2::TargetGroup",
    "AWS::ElasticLoadBalancingV2::TrustStore",
    "AWS::ElasticLoadBalancingV2::TrustStoreRevocation",
    "AWS::Elasticsearch::Domain",
    "AWS::Events::EventBus",
    "AWS::Events::EventBusPolicy",
    "AWS::Events::Rule",
    "AWS::FMS::NotificationChannel",
    "AWS::FMS::Policy",
    "AWS::FSx::DataRepositoryAssociation",
    "AWS::FSx::FileSystem",
    "AWS::FSx::Snapshot",
    "AWS::FSx::StorageVirtualMachine",
    "AWS::FSx::Volume",
    "AWS::GameLift::Alias",
    "AWS::GameLift::Build",
    "AWS::GameLift::Fleet",
    "AWS::GlobalAccelerator::Accelerator",
    "AWS::GlobalAccelerator::CrossAccountAttachment",
    "AWS::GlobalAccelerator::EndpointGroup",
    "AWS::GlobalAccelerator::Listener",
    "AWS::Glue::Classifier",
    "AWS::Glue::Connection",
    "AWS::Glue::Crawler",
    "AWS::Glue::CustomEntityType",
    "AWS::Glue::DataCatalogEncryptionSettings",
    "AWS::Glue::DataQualityRuleset",
    "AWS::Glue::Database",
    "AWS::Glue::DevEndpoint",
    "AWS::Glue::Job",
    "AWS::Glue::MLTransform",
    "AWS::Glue::Partition",
    "AWS::Glue::SecurityConfiguration",
    "AWS::Glue::Table",
    "AWS::Glue::Trigger",
    "AWS::Glue::Workflow",
    "AWS::GuardDuty::Detector",
    "AWS::GuardDuty::Filter",
    "AWS::GuardDuty::IPSet",
    "AWS::GuardDuty::MalwareProtectionPlan",
    "AWS::GuardDuty::Master",
    "AWS::GuardDuty::Member",
    "AWS::GuardDuty::ThreatIntelSet",
    "AWS::IAM::AccessKey",
    "AWS::IAM::Group",
    "AWS::IAM::InstanceProfile",
    "AWS::IAM::ManagedPolicy",
    "AWS::IAM::OIDCProvider",
    "AWS::IAM::Policy",
    "AWS::IAM::Role",
    "AWS::IAM::RolePolicy",
    "AWS::IAM::SAMLProvider",
    "AWS::IAM::ServerCertificate",
    "AWS::IAM::ServiceLinkedRole",
    "AWS::IAM::User",
    "AWS::IAM::UserToGroupAddition",
    "AWS::IdentityStore::Group",
    "AWS::IdentityStore::GroupMembership",
    "AWS::ImageBuilder::Component",
    "AWS::ImageBuilder::ContainerRecipe",
    "AWS::ImageBuilder::DistributionConfiguration",
    "AWS::ImageBuilder::Image",
    "AWS::ImageBuilder::ImagePipeline",
    "AWS::ImageBuilder::ImageRecipe",
    "AWS::ImageBuilder::InfrastructureConfiguration",
    "AWS::ImageBuilder::Workflow",
    "AWS::IoT::Certificate",
    "AWS::IoT::Policy",
    "AWS::IoT::PolicyPrincipalAttachment",
    "AWS::IoT::Thing",
    "AWS::IoT::ThingPrincipalAttachment",
    "AWS::IoT::TopicRule",
    "AWS::KMS::Alias",
    "AWS::KMS::Key",
    "AWS::KMS::ReplicaKey",
    "AWS::Kinesis::Stream",
    "AWS::Kinesis::StreamConsumer",
    "AWS::KinesisAnalyticsV2::Application",
    "AWS::KinesisFirehose::DeliveryStream",
    "AWS::LakeFormation::DataCellsFilter",
    "AWS::LakeFormation::DataLakeSettings",
    "AWS::LakeFormation::Permissions",
    "AWS::LakeFormation::PrincipalPermissions",
    "AWS::LakeFormation::Resource",
    "AWS::LakeFormation::Tag",
    "AWS::LakeFormation::TagAssociation",
    "AWS::Lambda::Alias",
    "AWS::Lambda::EventInvokeConfig",
    "AWS::Lambda::EventSourceMapping",
    "AWS::Lambda::Function",
    "AWS::Lambda::LayerVersion",
    "AWS::Lambda::LayerVersionPermission",
    "AWS::Lambda::Permission",
    "AWS::Lambda::Version",
    "AWS::Logs::AccountPolicy",
    "AWS::Logs::Delivery",
    "AWS::Logs::DeliveryDestination",
    "AWS::Logs::DeliverySource",
    "AWS::Logs::Destination",
    "AWS::Logs::LogGroup",
    "AWS::Logs::LogStream",
    "AWS::Logs::MetricFilter",
    "AWS::Logs::QueryDefinition",
    "AWS::Logs::ResourcePolicy",
    "AWS::Logs::SubscriptionFilter",
    "AWS::M2::Application",
    "AWS::M2::Environment",
    "AWS::MSK::BatchScramSecret",
    "AWS::MSK::Cluster",
    "AWS::MSK::ClusterPolicy",
    "AWS::MSK::Configuration",
    "AWS::MSK::VpcConnection",
    "AWS::MWAA::Environment",
    "AWS::Macie::AllowList",
    "AWS::Macie::CustomDataIdentifier",
    "AWS::Macie::FindingsFilter",
    "AWS::Macie::Session",
    "AWS::Neptune::DBCluster",
    "AWS::Neptune::DBClusterParameterGroup",
    "AWS::Neptune::DBInstance",
    "AWS::Neptune::DBParameterGroup",
    "AWS::Neptune::DBSubnetGroup",
    "AWS::Neptune::EventSubscription",
    "AWS::NetworkFirewall::Firewall",
    "AWS::NetworkFirewall::FirewallPolicy",
    "AWS::NetworkFirewall::LoggingConfiguration",
    "AWS::NetworkFirewall::RuleGroup",
    "AWS::NetworkFirewall::TLSInspectionConfiguration",
    "AWS::NetworkManager::CustomerGatewayAssociation",
    "AWS::NetworkManager::Device",
    "AWS::NetworkManager::GlobalNetwork",
    "AWS::NetworkManager::Link",
    "AWS::NetworkManager::LinkAssociation",
    "AWS::NetworkManager::Site",
    "AWS::NetworkManager::TransitGatewayRegistration",
    "AWS::Oam::Link",
    "AWS::Oam::Sink",
    "AWS::Omics::AnnotationStore",
    "AWS::Omics::ReferenceStore",
    "AWS::Omics::RunGroup",
    "AWS::Omics::SequenceStore",
    "AWS::Omics::VariantStore",
    "AWS::Omics::Workflow",
    "AWS::OpenSearchService::Domain",
    "AWS::OpsWorks::App",
    "AWS::OpsWorks::ElasticLoadBalancerAttachment",
    "AWS::OpsWorks::Instance",
    "AWS::OpsWorks::Layer",
    "AWS::OpsWorks::Stack",
    "AWS::OpsWorks::UserProfile",
    "AWS::OpsWorks::Volume",
    "AWS::Organizations::Account",
    "AWS::Organizations::Organization",
    "AWS::Organizations::OrganizationalUnit",
    "AWS::Organizations::Policy",
    "AWS::Organizations::ResourcePolicy",
    "AWS::RAM::Permission",
    "AWS::RAM::ResourceShare",
    "AWS::RDS::DBCluster",
    "AWS::RDS::DBClusterParameterGroup",
    "AWS::RDS::DBInstance",
    "AWS::RDS::DBParameterGroup",
    "AWS::RDS::DBProxy",
    "AWS::RDS::DBProxyEndpoint",
    "AWS::RDS::DBProxyTargetGroup",
    "AWS::RDS::DBSecurityGroup",
    "AWS::RDS::DBSecurityGroupIngress",
    "AWS::RDS::DBSubnetGroup",
    "AWS::RDS::EventSubscription",
    "AWS::RDS::GlobalCluster",
    "AWS::RDS::OptionGroup",
    "AWS::Redshift::Cluster",
    "AWS::Redshift::ClusterParameterGroup",
    "AWS::Redshift::ClusterSecurityGroup",
    "AWS::Redshift::ClusterSecurityGroupIngress",
    "AWS::Redshift::ClusterSubnetGroup",
    "AWS::Redshift::EndpointAccess",
    "AWS::Redshift::EndpointAuthorization",
    "AWS::Redshift::EventSubscription",
    "AWS::Redshift::ScheduledAction",
    "AWS::Rekognition::Collection",
    "AWS::ResourceExplorer2::DefaultViewAssociation",
    "AWS::ResourceExplorer2::Index",
    "AWS::ResourceExplorer2::View",
    "AWS::ResourceGroups::Group",
    "AWS::RolesAnywhere::CRL",
    "AWS::RolesAnywhere::Profile",
    "AWS::RolesAnywhere::TrustAnchor",
    "AWS::Route53::DNSSEC",
    "AWS::Route53::HealthCheck",
    "AWS::Route53::HostedZone",
    "AWS::Route53::KeySigningKey",
    "AWS::Route53::RecordSet",
    "AWS::Route53::RecordSetGroup",
    "AWS::Route53Profiles::Profile",
    "AWS::Route53Profiles::ProfileAssociation",
    "AWS::Route53Profiles::ProfileResourceAssociation",
    "AWS::Route53Resolver::FirewallDomainList",
    "AWS::Route53Resolver::FirewallRuleGroup",
    "AWS::Route53Resolver::FirewallRuleGroupAssociation",
    "AWS::Route53Resolver::OutpostResolver",
    "AWS::Route53Resolver::ResolverConfig",
    "AWS::Route53Resolver::ResolverDNSSECConfig",
    "AWS::Route53Resolver::ResolverEndpoint",
    "AWS::Route53Resolver::ResolverQueryLoggingConfig",
    "AWS::Route53Resolver::ResolverQueryLoggingConfigAssociation",
    "AWS::Route53Resolver::ResolverRule",
    "AWS::Route53Resolver::ResolverRuleAssociation",
    "AWS::S3::AccessGrant",
    "AWS::S3::AccessGrantsInstance",
    "AWS::S3::AccessGrantsLocation",
    "AWS::S3::AccessPoint",
    "AWS::S3::Bucket",
    "AWS::S3::BucketPolicy",
    "AWS::S3ObjectLambda::AccessPoint",
    "AWS::S3ObjectLambda::AccessPointPolicy",
    "AWS::S3Outposts::AccessPoint",
    "AWS::S3Outposts::Bucket",
    "AWS::S3Outposts::BucketPolicy",
    "AWS::S3Outposts::Endpoint",
    "AWS::SDB::Domain",
    "AWS::SES::ConfigurationSet",
    "AWS::SES::ConfigurationSetEventDestination",
    "AWS::SES::ContactList",
    "AWS::SES::DedicatedIpPool",
    "AWS::SES::EmailIdentity",
    "AWS::SES::Template",
    "AWS::SES::VdmAttributes",
    "AWS::SNS::Subscription",
    "AWS::SNS::Topic",
    "AWS::SNS::TopicInlinePolicy",
    "AWS::SNS::TopicPolicy",
    "AWS::SQS::Queue",
    "AWS::SQS::QueueInlinePolicy",
    "AWS::SQS::QueuePolicy",
    "AWS::SSM::Association",
    "AWS::SSM::Document",
    "AWS::SSM::MaintenanceWindow",
    "AWS::SSM::MaintenanceWindowTarget",
    "AWS::SSM::MaintenanceWindowTask",
    "AWS::SSM::Parameter",
    "AWS::SSM::PatchBaseline",
    "AWS::SSO::Application",
    "AWS::SSO::ApplicationAssignment",
    "AWS::SSO::Assignment",
    "AWS::SSO::Instance",
    "AWS::SSO::InstanceAccessControlAttributeConfiguration",
    "AWS::SSO::PermissionSet",
    "AWS::SageMaker::App",
    "AWS::SageMaker::AppImageConfig",
    "AWS::SageMaker::Domain",
    "AWS::SageMaker::FeatureGroup",
    "AWS::SageMaker::Image",
    "AWS::SageMaker::ImageVersion",
    "AWS::SageMaker::MlflowTrackingServer",
    "AWS::SageMaker::ModelCard",
    "AWS::SageMaker::ModelPackage",
    "AWS::SageMaker::ModelPackageGroup",
    "AWS::SageMaker::Pipeline",
    "AWS::SageMaker::Project",
    "AWS::SageMaker::UserProfile",
    "AWS::SecretsManager::ResourcePolicy",
    "AWS::SecretsManager::RotationSchedule",
    "AWS::SecretsManager::Secret",
    "AWS::SecretsManager::SecretTargetAttachment",
    "AWS::SecurityHub::AutomationRule",
    "AWS::SecurityHub::DelegatedAdmin",
    "AWS::SecurityHub::Hub",
    "AWS::SecurityHub::Insight",
    "AWS::SecurityHub::OrganizationConfiguration",
    "AWS::SecurityHub::ProductSubscription",
    "AWS::SecurityHub::SecurityControl",
    "AWS::SecurityHub::Standard",
    "AWS::ServiceCatalog::AcceptedPortfolioShare",
    "AWS::ServiceCatalog::CloudFormationProduct",
    "AWS::ServiceCatalog::CloudFormationProvisionedProduct",
    "AWS::ServiceCatalog::LaunchNotificationConstraint",
    "AWS::ServiceCatalog::LaunchRoleConstraint",
    "AWS::ServiceCatalog::LaunchTemplateConstraint",
    "AWS::ServiceCatalog::Portfolio",
    "AWS::ServiceCatalog::PortfolioPrincipalAssociation",
    "AWS::ServiceCatalog::PortfolioProductAssociation",
    "AWS::ServiceCatalog::PortfolioShare",
    "AWS::ServiceCatalog::ResourceUpdateConstraint",
    "AWS::ServiceCatalog::ServiceAction",
    "AWS::ServiceCatalog::ServiceActionAssociation",
    "AWS::ServiceCatalog::StackSetConstraint",
    "AWS::ServiceCatalog::TagOption",
    "AWS::ServiceCatalog::TagOptionAssociation",
    "AWS::ServiceCatalogAppRegistry::Application",
    "AWS::ServiceCatalogAppRegistry::AttributeGroup",
    "AWS::ServiceCatalogAppRegistry::AttributeGroupAssociation",
    "AWS::ServiceCatalogAppRegistry::ResourceAssociation",
    "AWS::ServiceDiscovery::HttpNamespace",
    "AWS::ServiceDiscovery::Instance",
    "AWS::ServiceDiscovery::PrivateDnsNamespace",
    "AWS::ServiceDiscovery::PublicDnsNamespace",
    "AWS::ServiceDiscovery::Service",
    "AWS::Shield::DRTAccess",
    "AWS::Shield::ProactiveEngagement",
    "AWS::Shield::Protection",
    "AWS::Shield::ProtectionGroup",
    "AWS::StepFunctions::Activity",
    "AWS::StepFunctions::StateMachine",
    "AWS::StepFunctions::StateMachineAlias",
    "AWS::StepFunctions::StateMachineVersion",
    "AWS::Synthetics::Canary",
    "AWS::Transfer::Agreement",
    "AWS::Transfer::Certificate",
    "AWS::Transfer::Connector",
    "AWS::Transfer::Profile",
    "AWS::Transfer::Server",
    "AWS::Transfer::User",
    "AWS::Transfer::Workflow",
    "AWS::VerifiedPermissions::IdentitySource",
    "AWS::VerifiedPermissions::Policy",
    "AWS::VerifiedPermissions::PolicyStore",
    "AWS::VerifiedPermissions::PolicyTemplate",
    "AWS::WAF::ByteMatchSet",
    "AWS::WAF::IPSet",
    "AWS::WAF::Rule",
    "AWS::WAF::SizeConstraintSet",
    "AWS::WAF::SqlInjectionMatchSet",
    "AWS::WAF::WebACL",
    "AWS::WAF::XssMatchSet",
    "AWS::WAFv2::IPSet",
    "AWS::WAFv2::LoggingConfiguration",
    "AWS::WAFv2::RegexPatternSet",
    "AWS::WAFv2::RuleGroup",
    "AWS::WAFv2::WebACL",
    "AWS::WAFv2::WebACLAssociation",
    "AWS::WorkSpaces::Workspace",
    "AWS::XRay::Group",
    "AWS::XRay::ResourcePolicy",
    "AWS::XRay::SamplingRule",
    "Module",
]

# pylint: disable=too-many-lines
cached: list[str] = [
    "Module",
    "aws-accessanalyzer-analyzer.json",
    "aws-acmpca-certificate.json",
    "aws-acmpca-certificateauthority.json",
    "aws-acmpca-certificateauthorityactivation.json",
    "aws-acmpca-permission.json",
    "aws-amazonmq-broker.json",
    "aws-amazonmq-configuration.json",
    "aws-apigatewayv2-apimapping.json",
    "aws-apigatewayv2-authorizer.json",
    "aws-apigatewayv2-deployment.json",
    "aws-apigatewayv2-domainname.json",
    "aws-apigatewayv2-integrationresponse.json",
    "aws-apigatewayv2-model.json",
    "aws-apigatewayv2-route.json",
    "aws-apigatewayv2-routeresponse.json",
    "aws-apigatewayv2-stage.json",
    "aws-appconfig-application.json",
    "aws-appconfig-configurationprofile.json",
    "aws-appconfig-deployment.json",
    "aws-appconfig-deploymentstrategy.json",
    "aws-appconfig-environment.json",
    "aws-appconfig-extension.json",
    "aws-appconfig-extensionassociation.json",
    "aws-appconfig-hostedconfigurationversion.json",
    "aws-applicationautoscaling-scalingpolicy.json",
    "aws-applicationsignals-servicelevelobjective.json",
    "aws-appmesh-gatewayroute.json",
    "aws-appmesh-mesh.json",
    "aws-appmesh-route.json",
    "aws-appmesh-virtualgateway.json",
    "aws-appmesh-virtualnode.json",
    "aws-appmesh-virtualrouter.json",
    "aws-appmesh-virtualservice.json",
    "aws-appsync-apicache.json",
    "aws-appsync-apikey.json",
    "aws-appsync-datasource.json",
    "aws-appsync-domainname.json",
    "aws-appsync-domainnameapiassociation.json",
    "aws-appsync-functionconfiguration.json",
    "aws-appsync-graphqlapi.json",
    "aws-appsync-graphqlschema.json",
    "aws-appsync-resolver.json",
    "aws-appsync-sourceapiassociation.json",
    "aws-arczonalshift-autoshiftobservernotificationstatus.json",
    "aws-arczonalshift-zonalautoshiftconfiguration.json",
    "aws-athena-datacatalog.json",
    "aws-athena-namedquery.json",
    "aws-athena-preparedstatement.json",
    "aws-athena-workgroup.json",
    "aws-autoscaling-autoscalinggroup.json",
    "aws-autoscaling-warmpool.json",
    "aws-backup-backupplan.json",
    "aws-backup-backupselection.json",
    "aws-backup-backupvault.json",
    "aws-batch-computeenvironment.json",
    "aws-batch-jobdefinition.json",
    "aws-batch-jobqueue.json",
    "aws-batch-schedulingpolicy.json",
    "aws-ce-anomalymonitor.json",
    "aws-ce-anomalysubscription.json",
    "aws-certificatemanager-certificate.json",
    "aws-chatbot-microsoftteamschannelconfiguration.json",
    "aws-chatbot-slackchannelconfiguration.json",
    "aws-cloud9-environmentec2.json",
    "aws-cloudformation-customresource.json",
    "aws-cloudformation-hookdefaultversion.json",
    "aws-cloudformation-hooktypeconfig.json",
    "aws-cloudformation-hookversion.json",
    "aws-cloudformation-macro.json",
    "aws-cloudformation-publictypeversion.json",
    "aws-cloudformation-resourcedefaultversion.json",
    "aws-cloudformation-resourceversion.json",
    "aws-cloudformation-stack.json",
    "aws-cloudformation-stackset.json",
    "aws-cloudformation-waitconditionhandle.json",
    "aws-cloudfront-cachepolicy.json",
    "aws-cloudfront-cloudfrontoriginaccessidentity.json",
    "aws-cloudfront-continuousdeploymentpolicy.json",
    "aws-cloudfront-distribution.json",
    "aws-cloudfront-function.json",
    "aws-cloudfront-keygroup.json",
    "aws-cloudfront-keyvaluestore.json",
    "aws-cloudfront-monitoringsubscription.json",
    "aws-cloudfront-originaccesscontrol.json",
    "aws-cloudfront-originrequestpolicy.json",
    "aws-cloudfront-publickey.json",
    "aws-cloudfront-realtimelogconfig.json",
    "aws-cloudfront-responseheaderspolicy.json",
    "aws-cloudtrail-eventdatastore.json",
    "aws-cloudtrail-trail.json",
    "aws-cloudwatch-alarm.json",
    "aws-cloudwatch-anomalydetector.json",
    "aws-cloudwatch-compositealarm.json",
    "aws-cloudwatch-dashboard.json",
    "aws-cloudwatch-insightrule.json",
    "aws-cloudwatch-metricstream.json",
    "aws-codebuild-project.json",
    "aws-codebuild-reportgroup.json",
    "aws-codebuild-sourcecredential.json",
    "aws-codedeploy-application.json",
    "aws-codedeploy-deploymentconfig.json",
    "aws-codedeploy-deploymentgroup.json",
    "aws-codepipeline-pipeline.json",
    "aws-cognito-identitypool.json",
    "aws-cognito-identitypoolprincipaltag.json",
    "aws-cognito-identitypoolroleattachment.json",
    "aws-cognito-logdeliveryconfiguration.json",
    "aws-cognito-userpool.json",
    "aws-cognito-userpoolclient.json",
    "aws-cognito-userpooldomain.json",
    "aws-cognito-userpoolgroup.json",
    "aws-cognito-userpoolidentityprovider.json",
    "aws-cognito-userpoolresourceserver.json",
    "aws-cognito-userpoolriskconfigurationattachment.json",
    "aws-cognito-userpooluicustomizationattachment.json",
    "aws-cognito-userpooluser.json",
    "aws-cognito-userpoolusertogroupattachment.json",
    "aws-config-aggregationauthorization.json",
    "aws-config-configrule.json",
    "aws-config-configurationaggregator.json",
    "aws-config-configurationrecorder.json",
    "aws-config-deliverychannel.json",
    "aws-config-organizationconfigrule.json",
    "aws-config-remediationconfiguration.json",
    "aws-config-storedquery.json",
    "aws-controltower-enabledbaseline.json",
    "aws-controltower-enabledcontrol.json",
    "aws-controltower-landingzone.json",
    "aws-datasync-agent.json",
    "aws-datasync-locationazureblob.json",
    "aws-datasync-locationefs.json",
    "aws-datasync-locationfsxlustre.json",
    "aws-datasync-locationfsxontap.json",
    "aws-datasync-locationfsxwindows.json",
    "aws-datasync-locationhdfs.json",
    "aws-datasync-locationnfs.json",
    "aws-datasync-locationobjectstorage.json",
    "aws-datasync-locations3.json",
    "aws-datasync-locationsmb.json",
    "aws-datasync-task.json",
    "aws-detective-graph.json",
    "aws-detective-memberinvitation.json",
    "aws-detective-organizationadmin.json",
    "aws-dlm-lifecyclepolicy.json",
    "aws-dms-dataprovider.json",
    "aws-dms-instanceprofile.json",
    "aws-dms-migrationproject.json",
    "aws-dms-replicationconfig.json",
    "aws-dynamodb-globaltable.json",
    "aws-dynamodb-table.json",
    "aws-ec2-capacityreservation.json",
    "aws-ec2-capacityreservationfleet.json",
    "aws-ec2-clientvpnendpoint.json",
    "aws-ec2-clientvpnroute.json",
    "aws-ec2-clientvpntargetnetworkassociation.json",
    "aws-ec2-customergateway.json",
    "aws-ec2-dhcpoptions.json",
    "aws-ec2-ec2fleet.json",
    "aws-ec2-egressonlyinternetgateway.json",
    "aws-ec2-eip.json",
    "aws-ec2-eipassociation.json",
    "aws-ec2-flowlog.json",
    "aws-ec2-gatewayroutetableassociation.json",
    "aws-ec2-instance.json",
    "aws-ec2-instanceconnectendpoint.json",
    "aws-ec2-internetgateway.json",
    "aws-ec2-ipam.json",
    "aws-ec2-ipamallocation.json",
    "aws-ec2-ipampool.json",
    "aws-ec2-ipampoolcidr.json",
    "aws-ec2-ipamresourcediscovery.json",
    "aws-ec2-ipamresourcediscoveryassociation.json",
    "aws-ec2-ipamscope.json",
    "aws-ec2-keypair.json",
    "aws-ec2-launchtemplate.json",
    "aws-ec2-localgatewayroute.json",
    "aws-ec2-localgatewayroutetable.json",
    "aws-ec2-localgatewayroutetablevirtualinterfacegroupassociation.json",
    "aws-ec2-localgatewayroutetablevpcassociation.json",
    "aws-ec2-natgateway.json",
    "aws-ec2-networkacl.json",
    "aws-ec2-networkinterface.json",
    "aws-ec2-networkinterfaceattachment.json",
    "aws-ec2-placementgroup.json",
    "aws-ec2-prefixlist.json",
    "aws-ec2-route.json",
    "aws-ec2-routetable.json",
    "aws-ec2-securitygroup.json",
    "aws-ec2-securitygroupegress.json",
    "aws-ec2-securitygroupingress.json",
    "aws-ec2-snapshotblockpublicaccess.json",
    "aws-ec2-subnet.json",
    "aws-ec2-subnetcidrblock.json",
    "aws-ec2-subnetnetworkaclassociation.json",
    "aws-ec2-subnetroutetableassociation.json",
    "aws-ec2-trafficmirrorfilter.json",
    "aws-ec2-trafficmirrorfilterrule.json",
    "aws-ec2-trafficmirrortarget.json",
    "aws-ec2-transitgateway.json",
    "aws-ec2-transitgatewayconnect.json",
    "aws-ec2-transitgatewaymulticastdomain.json",
    "aws-ec2-transitgatewaymulticastdomainassociation.json",
    "aws-ec2-transitgatewaymulticastgroupmember.json",
    "aws-ec2-transitgatewaymulticastgroupsource.json",
    "aws-ec2-transitgatewaypeeringattachment.json",
    "aws-ec2-transitgatewayroute.json",
    "aws-ec2-transitgatewayroutetable.json",
    "aws-ec2-transitgatewayroutetableassociation.json",
    "aws-ec2-transitgatewayroutetablepropagation.json",
    "aws-ec2-verifiedaccessendpoint.json",
    "aws-ec2-verifiedaccessgroup.json",
    "aws-ec2-verifiedaccessinstance.json",
    "aws-ec2-verifiedaccesstrustprovider.json",
    "aws-ec2-volumeattachment.json",
    "aws-ec2-vpc.json",
    "aws-ec2-vpccidrblock.json",
    "aws-ec2-vpcdhcpoptionsassociation.json",
    "aws-ec2-vpcendpointconnectionnotification.json",
    "aws-ec2-vpcendpointservice.json",
    "aws-ec2-vpcendpointservicepermissions.json",
    "aws-ec2-vpcgatewayattachment.json",
    "aws-ec2-vpcpeeringconnection.json",
    "aws-ec2-vpngateway.json",
    "aws-ec2-vpngatewayroutepropagation.json",
    "aws-ecr-pullthroughcacherule.json",
    "aws-ecr-registrypolicy.json",
    "aws-ecr-replicationconfiguration.json",
    "aws-ecr-repository.json",
    "aws-ecr-repositorycreationtemplate.json",
    "aws-ecs-capacityprovider.json",
    "aws-ecs-clustercapacityproviderassociations.json",
    "aws-ecs-primarytaskset.json",
    "aws-ecs-service.json",
    "aws-ecs-taskdefinition.json",
    "aws-ecs-taskset.json",
    "aws-efs-accesspoint.json",
    "aws-efs-filesystem.json",
    "aws-efs-mounttarget.json",
    "aws-eks-accessentry.json",
    "aws-eks-addon.json",
    "aws-eks-cluster.json",
    "aws-eks-fargateprofile.json",
    "aws-eks-identityproviderconfig.json",
    "aws-eks-nodegroup.json",
    "aws-eks-podidentityassociation.json",
    "aws-elasticache-cachecluster.json",
    "aws-elasticache-parametergroup.json",
    "aws-elasticache-replicationgroup.json",
    "aws-elasticache-securitygroup.json",
    "aws-elasticache-securitygroupingress.json",
    "aws-elasticache-serverlesscache.json",
    "aws-elasticache-user.json",
    "aws-elasticache-usergroup.json",
    "aws-elasticloadbalancing-loadbalancer.json",
    "aws-elasticloadbalancingv2-listener.json",
    "aws-elasticloadbalancingv2-listenercertificate.json",
    "aws-elasticloadbalancingv2-listenerrule.json",
    "aws-elasticloadbalancingv2-loadbalancer.json",
    "aws-elasticloadbalancingv2-truststore.json",
    "aws-elasticloadbalancingv2-truststorerevocation.json",
    "aws-elasticsearch-domain.json",
    "aws-emr-cluster.json",
    "aws-emr-instancefleetconfig.json",
    "aws-emr-instancegroupconfig.json",
    "aws-emr-studio.json",
    "aws-emr-studiosessionmapping.json",
    "aws-events-eventbus.json",
    "aws-events-eventbuspolicy.json",
    "aws-events-rule.json",
    "aws-fms-notificationchannel.json",
    "aws-fsx-datarepositoryassociation.json",
    "aws-fsx-filesystem.json",
    "aws-fsx-snapshot.json",
    "aws-fsx-storagevirtualmachine.json",
    "aws-fsx-volume.json",
    "aws-globalaccelerator-accelerator.json",
    "aws-globalaccelerator-crossaccountattachment.json",
    "aws-globalaccelerator-endpointgroup.json",
    "aws-globalaccelerator-listener.json",
    "aws-glue-classifier.json",
    "aws-glue-connection.json",
    "aws-glue-crawler.json",
    "aws-glue-customentitytype.json",
    "aws-glue-database.json",
    "aws-glue-datacatalogencryptionsettings.json",
    "aws-glue-dataqualityruleset.json",
    "aws-glue-devendpoint.json",
    "aws-glue-job.json",
    "aws-glue-mltransform.json",
    "aws-glue-partition.json",
    "aws-glue-securityconfiguration.json",
    "aws-glue-table.json",
    "aws-glue-trigger.json",
    "aws-glue-workflow.json",
    "aws-guardduty-detector.json",
    "aws-guardduty-filter.json",
    "aws-guardduty-ipset.json",
    "aws-guardduty-malwareprotectionplan.json",
    "aws-guardduty-master.json",
    "aws-guardduty-member.json",
    "aws-guardduty-threatintelset.json",
    "aws-iam-group.json",
    "aws-iam-oidcprovider.json",
    "aws-iam-role.json",
    "aws-iam-rolepolicy.json",
    "aws-iam-samlprovider.json",
    "aws-iam-servercertificate.json",
    "aws-iam-servicelinkedrole.json",
    "aws-iam-user.json",
    "aws-iam-usertogroupaddition.json",
    "aws-identitystore-group.json",
    "aws-identitystore-groupmembership.json",
    "aws-imagebuilder-component.json",
    "aws-imagebuilder-containerrecipe.json",
    "aws-imagebuilder-distributionconfiguration.json",
    "aws-imagebuilder-image.json",
    "aws-imagebuilder-imagepipeline.json",
    "aws-imagebuilder-imagerecipe.json",
    "aws-imagebuilder-infrastructureconfiguration.json",
    "aws-imagebuilder-workflow.json",
    "aws-kinesis-stream.json",
    "aws-kinesisanalyticsv2-application.json",
    "aws-kms-alias.json",
    "aws-kms-key.json",
    "aws-kms-replicakey.json",
    "aws-lakeformation-datacellsfilter.json",
    "aws-lakeformation-datalakesettings.json",
    "aws-lakeformation-permissions.json",
    "aws-lakeformation-principalpermissions.json",
    "aws-lakeformation-resource.json",
    "aws-lakeformation-tag.json",
    "aws-lakeformation-tagassociation.json",
    "aws-lambda-alias.json",
    "aws-lambda-eventinvokeconfig.json",
    "aws-lambda-eventsourcemapping.json",
    "aws-lambda-layerversionpermission.json",
    "aws-lambda-permission.json",
    "aws-lambda-version.json",
    "aws-logs-accountpolicy.json",
    "aws-logs-delivery.json",
    "aws-logs-deliverydestination.json",
    "aws-logs-deliverysource.json",
    "aws-logs-destination.json",
    "aws-logs-loggroup.json",
    "aws-logs-logstream.json",
    "aws-logs-metricfilter.json",
    "aws-logs-querydefinition.json",
    "aws-logs-resourcepolicy.json",
    "aws-logs-subscriptionfilter.json",
    "aws-m2-application.json",
    "aws-m2-environment.json",
    "aws-macie-allowlist.json",
    "aws-macie-customdataidentifier.json",
    "aws-macie-findingsfilter.json",
    "aws-macie-session.json",
    "aws-msk-batchscramsecret.json",
    "aws-msk-cluster.json",
    "aws-msk-clusterpolicy.json",
    "aws-msk-configuration.json",
    "aws-msk-vpcconnection.json",
    "aws-mwaa-environment.json",
    "aws-neptune-dbsubnetgroup.json",
    "aws-neptune-eventsubscription.json",
    "aws-networkfirewall-firewall.json",
    "aws-networkfirewall-firewallpolicy.json",
    "aws-networkfirewall-loggingconfiguration.json",
    "aws-networkfirewall-rulegroup.json",
    "aws-networkfirewall-tlsinspectionconfiguration.json",
    "aws-networkmanager-customergatewayassociation.json",
    "aws-networkmanager-device.json",
    "aws-networkmanager-globalnetwork.json",
    "aws-networkmanager-link.json",
    "aws-networkmanager-linkassociation.json",
    "aws-networkmanager-site.json",
    "aws-networkmanager-transitgatewayregistration.json",
    "aws-oam-link.json",
    "aws-oam-sink.json",
    "aws-omics-annotationstore.json",
    "aws-omics-referencestore.json",
    "aws-omics-rungroup.json",
    "aws-omics-sequencestore.json",
    "aws-omics-variantstore.json",
    "aws-omics-workflow.json",
    "aws-opensearchservice-domain.json",
    "aws-opsworks-app.json",
    "aws-opsworks-elasticloadbalancerattachment.json",
    "aws-opsworks-userprofile.json",
    "aws-opsworks-volume.json",
    "aws-organizations-account.json",
    "aws-organizations-organization.json",
    "aws-organizations-organizationalunit.json",
    "aws-organizations-policy.json",
    "aws-organizations-resourcepolicy.json",
    "aws-ram-permission.json",
    "aws-ram-resourceshare.json",
    "aws-rds-dbcluster.json",
    "aws-rds-dbclusterparametergroup.json",
    "aws-rds-dbinstance.json",
    "aws-rds-dbparametergroup.json",
    "aws-rds-dbproxy.json",
    "aws-rds-dbproxyendpoint.json",
    "aws-rds-dbproxytargetgroup.json",
    "aws-rds-dbsecuritygroup.json",
    "aws-rds-dbsecuritygroupingress.json",
    "aws-rds-dbsubnetgroup.json",
    "aws-rds-eventsubscription.json",
    "aws-rds-globalcluster.json",
    "aws-rds-optiongroup.json",
    "aws-redshift-cluster.json",
    "aws-redshift-clusterparametergroup.json",
    "aws-redshift-clustersecuritygroup.json",
    "aws-redshift-clustersecuritygroupingress.json",
    "aws-redshift-clustersubnetgroup.json",
    "aws-redshift-endpointaccess.json",
    "aws-redshift-endpointauthorization.json",
    "aws-redshift-eventsubscription.json",
    "aws-redshift-scheduledaction.json",
    "aws-rekognition-collection.json",
    "aws-resourceexplorer2-defaultviewassociation.json",
    "aws-resourceexplorer2-index.json",
    "aws-resourceexplorer2-view.json",
    "aws-resourcegroups-group.json",
    "aws-rolesanywhere-crl.json",
    "aws-rolesanywhere-profile.json",
    "aws-rolesanywhere-trustanchor.json",
    "aws-route53-dnssec.json",
    "aws-route53-keysigningkey.json",
    "aws-route53-recordset.json",
    "aws-route53-recordsetgroup.json",
    "aws-route53profiles-profile.json",
    "aws-route53profiles-profileassociation.json",
    "aws-route53profiles-profileresourceassociation.json",
    "aws-route53resolver-firewalldomainlist.json",
    "aws-route53resolver-firewallrulegroupassociation.json",
    "aws-route53resolver-outpostresolver.json",
    "aws-route53resolver-resolverconfig.json",
    "aws-route53resolver-resolverdnssecconfig.json",
    "aws-route53resolver-resolverendpoint.json",
    "aws-route53resolver-resolverqueryloggingconfig.json",
    "aws-route53resolver-resolverqueryloggingconfigassociation.json",
    "aws-route53resolver-resolverrule.json",
    "aws-route53resolver-resolverruleassociation.json",
    "aws-s3-accessgrant.json",
    "aws-s3-accessgrantsinstance.json",
    "aws-s3-accessgrantslocation.json",
    "aws-s3-accesspoint.json",
    "aws-s3-bucket.json",
    "aws-s3-bucketpolicy.json",
    "aws-s3objectlambda-accesspoint.json",
    "aws-s3objectlambda-accesspointpolicy.json",
    "aws-s3outposts-accesspoint.json",
    "aws-s3outposts-bucket.json",
    "aws-s3outposts-bucketpolicy.json",
    "aws-s3outposts-endpoint.json",
    "aws-sagemaker-appimageconfig.json",
    "aws-sagemaker-featuregroup.json",
    "aws-sagemaker-image.json",
    "aws-sagemaker-imageversion.json",
    "aws-sagemaker-mlflowtrackingserver.json",
    "aws-sagemaker-modelcard.json",
    "aws-sagemaker-modelpackage.json",
    "aws-sagemaker-modelpackagegroup.json",
    "aws-sagemaker-pipeline.json",
    "aws-sagemaker-project.json",
    "aws-sdb-domain.json",
    "aws-secretsmanager-rotationschedule.json",
    "aws-secretsmanager-secret.json",
    "aws-securityhub-automationrule.json",
    "aws-securityhub-delegatedadmin.json",
    "aws-securityhub-hub.json",
    "aws-securityhub-insight.json",
    "aws-securityhub-organizationconfiguration.json",
    "aws-securityhub-productsubscription.json",
    "aws-securityhub-securitycontrol.json",
    "aws-securityhub-standard.json",
    "aws-servicecatalog-cloudformationproduct.json",
    "aws-servicecatalog-cloudformationprovisionedproduct.json",
    "aws-servicecatalog-launchnotificationconstraint.json",
    "aws-servicecatalog-launchroleconstraint.json",
    "aws-servicecatalog-launchtemplateconstraint.json",
    "aws-servicecatalog-portfolioshare.json",
    "aws-servicecatalog-resourceupdateconstraint.json",
    "aws-servicecatalog-serviceaction.json",
    "aws-servicecatalog-serviceactionassociation.json",
    "aws-servicecatalog-stacksetconstraint.json",
    "aws-servicecatalog-tagoption.json",
    "aws-servicecatalogappregistry-application.json",
    "aws-servicecatalogappregistry-attributegroup.json",
    "aws-servicecatalogappregistry-attributegroupassociation.json",
    "aws-servicecatalogappregistry-resourceassociation.json",
    "aws-servicediscovery-httpnamespace.json",
    "aws-servicediscovery-instance.json",
    "aws-servicediscovery-privatednsnamespace.json",
    "aws-servicediscovery-publicdnsnamespace.json",
    "aws-servicediscovery-service.json",
    "aws-ses-configurationset.json",
    "aws-ses-configurationseteventdestination.json",
    "aws-ses-contactlist.json",
    "aws-ses-dedicatedippool.json",
    "aws-ses-emailidentity.json",
    "aws-ses-template.json",
    "aws-ses-vdmattributes.json",
    "aws-shield-drtaccess.json",
    "aws-shield-proactiveengagement.json",
    "aws-shield-protection.json",
    "aws-shield-protectiongroup.json",
    "aws-sns-subscription.json",
    "aws-sns-topic.json",
    "aws-sns-topicinlinepolicy.json",
    "aws-sqs-queue.json",
    "aws-sqs-queueinlinepolicy.json",
    "aws-ssm-association.json",
    "aws-ssm-document.json",
    "aws-ssm-maintenancewindow.json",
    "aws-ssm-maintenancewindowtarget.json",
    "aws-ssm-maintenancewindowtask.json",
    "aws-ssm-parameter.json",
    "aws-ssm-patchbaseline.json",
    "aws-sso-application.json",
    "aws-sso-applicationassignment.json",
    "aws-sso-assignment.json",
    "aws-sso-instance.json",
    "aws-sso-instanceaccesscontrolattributeconfiguration.json",
    "aws-sso-permissionset.json",
    "aws-stepfunctions-activity.json",
    "aws-stepfunctions-statemachine.json",
    "aws-stepfunctions-statemachinealias.json",
    "aws-stepfunctions-statemachineversion.json",
    "aws-synthetics-canary.json",
    "aws-transfer-agreement.json",
    "aws-transfer-certificate.json",
    "aws-transfer-connector.json",
    "aws-transfer-profile.json",
    "aws-transfer-server.json",
    "aws-transfer-user.json",
    "aws-transfer-workflow.json",
    "aws-verifiedpermissions-identitysource.json",
    "aws-verifiedpermissions-policy.json",
    "aws-verifiedpermissions-policystore.json",
    "aws-verifiedpermissions-policytemplate.json",
    "aws-waf-bytematchset.json",
    "aws-waf-sqlinjectionmatchset.json",
    "aws-wafv2-ipset.json",
    "aws-wafv2-loggingconfiguration.json",
    "aws-wafv2-regexpatternset.json",
    "aws-wafv2-rulegroup.json",
    "aws-wafv2-webacl.json",
    "aws-wafv2-webaclassociation.json",
    "aws-workspaces-workspace.json",
    "aws-xray-group.json",
    "aws-xray-resourcepolicy.json",
    "aws-xray-samplingrule.json",
    "module.json",
]



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any


class UnknownSatisfisfaction(Exception):
    """Unknown Satisfisfaction Exception"""

    def __init__(self, message: str, **kwargs: Any):
        super().__init__(message, **kwargs)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import pytest

from cfnlint.helpers import is_types_compatible


@pytest.mark.parametrize(
    "name,source,destination,expected",
    [
        (
            "Types are directly working together",
            "string",
            ["string"],
            True,
        ),
        (
            "Types can be converted from source to destiniation",
            "string",
            ["integer"],
            True,
        ),
        (
            "Types can be converted from destination to source",
            "integer",
            ["string"],
            True,
        ),
        (
            "Types of integer against array are not compatible",
            "integer",
            "array",
            False,
        ),
        (
            "Types of integer can be compatible",
            "integer",
            "number",
            True,
        ),
        (
            "Types of number are not an integer",
            "number",
            "integer",
            False,
        ),
        (
            "Types of integer is not compatible with boolean",
            "integer",
            "boolean",
            False,
        ),
        (
            "Types of number is not compatible with boolean",
            "number",
            "boolean",
            False,
        ),
        (
            "Types of number, string is compatible with boolean",
            ["number", "string"],
            "boolean",
            True,
        ),
    ],
)
def test_validate(name, source, destination, expected):
    result = is_types_compatible(source, destination)
    assert result == expected, f"Test {name!r} got {result!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from io import StringIO
from test.testlib.testcase import BaseTestCase
from unittest.mock import patch

import cfnlint.decode.cfn_yaml  # pylint: disable=E0401
from cfnlint.config import _DEFAULT_RULESDIR, ConfigMixIn
from cfnlint.rules import Rules
from cfnlint.template.template import Template  # pylint: disable=E0401


class TestYamlParse(BaseTestCase):
    """Test YAML Parsing"""

    def setUp(self):
        """SetUp template object"""
        self.rules = Rules.create_from_directory(_DEFAULT_RULESDIR)

        self.filenames = {
            "config_rule": {
                "filename": "test/fixtures/templates/public/lambda-poller.yaml",
                "failures": 1,
            },
            "generic_bad": {
                "filename": "test/fixtures/templates/bad/generic.yaml",
                "failures": 35,
            },
        }

    def test_success_parse(self):
        """Test Successful YAML Parsing"""
        for _, values in self.filenames.items():
            filename = values.get("filename")
            failures = values.get("failures")
            template = cfnlint.decode.cfn_yaml.load(filename)
            cfn = Template(filename, template, ["us-east-1"])

            matches = list(self.rules.run(filename, cfn, ConfigMixIn({})))
            assert (
                len(matches) == failures
            ), "Expected {} failures, got {} on {}".format(
                failures, len(matches), filename
            )

    def test_success_parse_stdin(self):
        """Test Successful YAML Parsing through stdin"""
        for _, values in self.filenames.items():
            filename = None
            failures = values.get("failures")
            with open(values.get("filename"), "r") as fp:
                file_content = fp.read()

            with patch("sys.stdin", StringIO(file_content)):
                template = cfnlint.decode.cfn_yaml.load(filename)
                cfn = Template(filename, template, ["us-east-1"])

                matches = []
                matches.extend(self.rules.run(filename, cfn, ConfigMixIn({})))
                assert (
                    len(matches) == failures
                ), "Expected {} failures, got {} on {}".format(
                    failures, len(matches), values.get("filename")
                )

    def test_map_failure(self):
        """Test a failure is passed on unhashable map"""
        filename = "test/fixtures/templates/bad/core/parse_invalid_map.yaml"

        self.assertRaises(
            cfnlint.decode.cfn_yaml.CfnParseError,
            cfnlint.decode.cfn_yaml.load,
            filename,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.jsonschema import ValidationResult
from cfnlint.jsonschema.protocols import Validator
from cfnlint.rules.functions._BaseFn import BaseFn


class Join(BaseFn):
    """Check if Join values are correct"""

    id = "E1022"
    shortdesc = "Join validation of parameters"
    description = "Making sure the join function is properly configured"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-join.html"
    tags = ["functions", "join"]

    def __init__(self) -> None:
        super().__init__("Fn::Join", ("string",), resolved_rule="W1032")
        self.child_rules.update(
            {
                "I1022": None,
            }
        )

    def schema(self, validator, instance) -> dict[str, Any]:
        return {
            "type": "array",
            "maxItems": 2,
            "minItems": 2,
            "fn_items": [
                {
                    "schema": {
                        "type": ["string"],
                    },
                },
                {
                    "functions": [
                        "Fn::Cidr",
                        "Fn::FindInMap",
                        "Fn::GetAtt",
                        "Fn::If",
                        "Fn::Split",
                        "Ref",
                    ],
                    "schema": {
                        "type": ["array"],
                        "fn_items": {
                            "functions": [
                                "Fn::Base64",
                                "Fn::FindInMap",
                                "Fn::GetAtt",
                                "Fn::If",
                                "Fn::ImportValue",
                                "Fn::Join",
                                "Fn::Select",
                                "Fn::Sub",
                                "Fn::Transform",
                                "Ref",
                            ],
                            "schema": {
                                "type": ["string"],
                            },
                        },
                    },
                },
            ],
        }

    def fn_join(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        validator = validator.evolve(
            context=validator.context.evolve(strict_types=True)
        )
        errs = list(super().validate(validator, s, instance, schema))
        if errs:
            yield from iter(errs)

        for rule in self.child_rules.values():
            if rule is None or not hasattr(rule, "validate"):
                continue

            yield from rule.validate(validator, s, instance, schema)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import cfnlint.data.schemas.extensions.aws_amazonmq_broker
from cfnlint.rules.jsonschema.CfnLintJsonSchema import SchemaDetails
from cfnlint.rules.jsonschema.CfnLintJsonSchemaRegional import CfnLintJsonSchemaRegional


class BrokerInstanceTypeEnum(CfnLintJsonSchemaRegional):
    id = "E3670"
    shortdesc = "Validate the instance types for an AmazonMQ Broker"
    description = (
        "Validates the instance types for AmazonMQ broker based on region "
        "and data gathered from the pricing APIs"
    )
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::AmazonMQ::Broker/Properties/HostInstanceType"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_amazonmq_broker,
                filename="instancetype_enum.json",
            ),
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.cloudwatch.AlarmAwsNamespacePeriod import (
    AlarmAwsNamespacePeriod,
)


@pytest.fixture(scope="module")
def rule():
    rule = AlarmAwsNamespacePeriod()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {
                "Namespace": "AWS/foo",
                "Period": 60,
            },
            [],
        ),
        (
            [],  # wrong type
            [],
        ),
        (
            {
                "Namespace": "AWS/foo",
                "Period": {"Ref": "Period"},  # ignore when object
            },
            [],
        ),
        (
            {
                "Namespace": {"Ref": "Namespace"},  # ignore when object
                "Period": 30,
            },
            [],
        ),
        (
            {
                "Namespace": "AWS/foo",
                "Period": 30,
            },
            [
                ValidationError(
                    "30 is less than the minimum of 60",
                    rule=AlarmAwsNamespacePeriod(),
                    path=deque(["Period"]),
                    validator="minimum",
                    schema={"minimum": 60, "type": ["number", "string"]},
                    schema_path=deque(["then", "properties", "Period", "minimum"]),
                )
            ],
        ),
    ],
)
def test_validate(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "", instance, {}))

    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.integration import BaseCliTestCase

from cfnlint import ConfigMixIn


class TestQuickStartTemplates(BaseCliTestCase):
    """Test QuickStart Templates Parsing"""

    scenarios = [
        {
            "filename": (
                "test/fixtures/templates/integration"
                "/resources-cloudformation-init.yaml"
            ),
            "results_filename": (
                "test/fixtures/results/integration/"
                "resources-cloudformation-init.json"
            ),
            "exit_code": 0,
        },
        {
            "filename": (
                "test/fixtures/templates/integration" "/dynamic-references.yaml"
            ),
            "results_filename": (
                "test/fixtures/results/integration/" "dynamic-references.json"
            ),
            "exit_code": 2,
        },
    ]

    def test_templates(self):
        """Test same templates using integration approach"""
        self.run_module_integration_scenarios(
            ConfigMixIn(
                [],
                include_checks=["I"],
                configure_rules={
                    "E3012": {
                        "strict": True,
                    }
                },
                include_experimental=True,
            )
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.properties.ImageId import ImageId


@pytest.fixture(scope="module")
def rule():
    rule = ImageId()
    yield rule


@pytest.fixture
def template():
    return {
        "Parameters": {
            "MyImageId": {
                "Type": "AWS::EC2::Image::Id",
            },
            "MyString": {
                "Type": "String",
            },
        },
        "Resources": {},
    }


@pytest.mark.parametrize(
    "name,instance,expected",
    [
        (
            "Valid Ref to a paraemter",
            {"Ref": "MyImageId"},
            [],
        ),
        (
            "Valid Ref to a Pseudo-Parameter",
            {"Ref": "AWS::Region"},
            [],
        ),
        (
            "Invalid Ref to a parameter of the wrong type",
            {"Ref": "MyString"},
            [
                ValidationError(
                    (
                        "'String' is not one of ['AWS::EC2::Image::Id'"
                        ", 'AWS::SSM::Parameter::Value<AWS::EC2::Image::Id>']"
                    ),
                    path=deque([]),
                    schema_path=deque(["enum"]),
                    validator="enum",
                    rule=ImageId(),
                    path_override=deque(["Parameters", "MyString", "Type"]),
                ),
            ],
        ),
    ],
)
def test_validate(name, instance, expected, rule, validator):
    errs = list(rule.validate(validator, {}, instance, {}))
    assert errs == expected, f"Test {name!r} got {errs!r}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.ecs.TaskFargateProperties import TaskFargateProperties


@pytest.fixture(scope="module")
def rule():
    rule = TaskFargateProperties()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Cpu": 256,
                "Memory": "512",
            },
            [],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Cpu": ".25     vCpU",
                "Memory": "512",
            },
            [],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Cpu": 16384,
            },
            [
                ValidationError(
                    "'Memory' is a required property",
                    rule=TaskFargateProperties(),
                    path=deque([]),
                    validator="required",
                    schema_path=deque(["then", "required"]),
                )
            ],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Memory": "512",
            },
            [
                ValidationError(
                    "'Cpu' is a required property",
                    rule=TaskFargateProperties(),
                    path=deque([]),
                    validator="required",
                    schema_path=deque(["then", "required"]),
                )
            ],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Memory": "512",
                "Cpu": 256,
                "PlacementConstraints": "foo",
            },
            [
                ValidationError(
                    ("'PlacementConstraints' isn't supported for Fargate tasks"),
                    rule=TaskFargateProperties(),
                    path=deque(["PlacementConstraints"]),
                    validator="not",
                    schema_path=deque(["then", "not"]),
                )
            ],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Cpu": {"Ref": "MyParameter"},
                "Memory": "512",
            },
            [],
        ),
        (
            {
                "RequiresCompatibilities": ["FARGATE"],
                "Cpu": 128,
                "Memory": "512",
            },
            [
                ValidationError(
                    (
                        "128 is not one of ['256', '512', '1024', '2048', "
                        "'4096', '8192', '16384']"
                    ),
                    rule=TaskFargateProperties(),
                    path=deque(["Cpu"]),
                    validator="enum",
                    schema_path=deque(
                        ["then", "then", "properties", "Cpu", "then", "enum"]
                    ),
                )
            ],
        ),
    ],
)
def test_validate(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "", instance, {}))
    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule


class PrefixItems(CloudFormationLintRule):
    """Check Required Resource Configuration"""

    id = "E3008"
    shortdesc = "Validate an array in order"
    description = "Will validate arrays in order for schema validation"
    source_url = "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#prefixitems"
    tags = ["resources", "properties", "array", "prefixItems"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import cfnlint.data.schemas.other.transforms
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class Configuration(CfnLintJsonSchema):
    """Check if Parameters are configured correctly"""

    id = "E1005"
    shortdesc = "Validate Transform configuration"
    description = (
        "Validate that the transforms section of a template is properly configured"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-section-structure.html"
    tags = ["transform"]

    def __init__(self):
        """Init"""
        super().__init__(
            keywords=["Transform"],
            schema_details=SchemaDetails(
                cfnlint.data.schemas.other.transforms, "configuration.json"
            ),
            all_matches=True,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import List

from cfnlint.config import ConfigMixIn, ManualArgs
from cfnlint.decode.decode import decode_str
from cfnlint.helpers import REGION_PRIMARY, REGIONS
from cfnlint.rules import Match, RulesCollection
from cfnlint.runner import Runner, TemplateRunner

Matches = List[Match]


def lint(
    s: str,
    rules: RulesCollection | None = None,
    regions: list[str] | None = None,
    config: ManualArgs | None = None,
) -> list[Match]:
    """Validate a string template using the specified rules and regions.

    Parameters
    ----------
    s : str
        the template string
    rules : RulesCollection
        The rules to run against s
    regions : list[str]
        The regions to test against s

    Returns
    -------
    list
        a list of errors if any were found, else an empty list
    """
    template, errors = decode_str(s)
    if errors:
        return errors

    if template is None:
        return []

    if not regions:
        regions = [REGION_PRIMARY]

    if not config:
        config_mixin = ConfigMixIn(
            regions=regions,
        )
    else:
        config_mixin = ConfigMixIn(**config)

    if isinstance(rules, RulesCollection):
        template_runner = TemplateRunner(None, template, config_mixin, rules)  # type: ignore # noqa: E501
        return list(template_runner.run())

    runner = Runner(config_mixin)
    return list(runner.validate_template(None, template))


def lint_all(s: str) -> list[Match]:
    """Validate a string template against all regions and rules.

    Parameters
    ----------
    s : str
        the template string

    Returns
    -------
    list
        a list of errors if any were found, else an empty list
    """
    return lint(
        s=s,
        config=ManualArgs(
            include_checks=["I"], include_experimental=True, regions=REGIONS
        ),
    )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from collections import deque
from typing import Any, Sequence

from cfnlint.context import Path
from cfnlint.context.conditions import Unsatisfiable
from cfnlint.jsonschema import Validator


def get_resource_by_name(
    validator: Validator, name: str, types: Sequence[str] | None = None
) -> tuple[Any, Validator]:

    resource = validator.context.resources.get(name)
    if not resource:
        return None, validator

    if types and resource.type not in types:
        return None, validator

    if resource.condition:
        try:
            validator = validator.evolve(
                context=validator.context.evolve(
                    conditions=validator.context.conditions.evolve(
                        {
                            resource.condition: True,
                        }
                    ),
                )
            )
        except Unsatisfiable:
            return None, validator

    validator = validator.evolve(
        context=validator.context.evolve(
            path=Path(
                path=deque(["Resources", name]),
                cfn_path=deque(["Resources", resource.type]),
            )
        )
    )

    return validator.cfn.template.get("Resources", {}).get(name), validator



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

from cfnlint.helpers import PSEUDOPARAMS
from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.functions._BaseFn import BaseFn


class ToJsonString(BaseFn):
    """Check if ToJsonString values are correct"""

    id = "E1031"
    shortdesc = "ToJsonString validation of parameters"
    description = "Making sure Fn::ToJsonString is configured correctly"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference.html"
    tags = ["functions", "toJsonString"]

    def __init__(self) -> None:
        super().__init__(
            "Fn::ToJsonString",
            ("string",),
            (
                "Fn::FindInMap",
                "Fn::GetAtt",
                "Fn::GetAZs",
                "Fn::If",
                "Fn::Select",
                "Fn::Split",
                "Ref",
            ),
            resolved_rule="W1040",
        )

    def schema(self, validator: Validator, instance: Any) -> dict[str, Any]:
        return {
            "type": ["array", "object"],
            "minItems": 1,
            "minProperties": 1,
        }

    def validator(self, validator: Validator) -> Validator:
        return validator.evolve(
            context=validator.context.evolve(
                functions=self.functions,
                pseudo_parameters=set(
                    [sp for sp in PSEUDOPARAMS if sp != "AWS::NotificationARNs"]
                ),
            ),
            function_filter=validator.function_filter.evolve(
                add_cfn_lint_keyword=False,
            ),
        )

    def fn_tojsonstring(
        self, validator: Validator, s: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        if not validator.context.transforms.has_language_extensions_transform():
            yield ValidationError(
                (
                    f"{self.fn.name} is not supported without "
                    "'AWS::LanguageExtensions' transform"
                ),
                validator=self.fn.py,
                rule=self,
            )
            return

        for err in super().validate(validator, s, instance, schema):
            err.rule = self
            yield err



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import regex as re

from cfnlint.jsonschema import Validator
from cfnlint.rules.formats.FormatKeyword import FormatKeyword


class SecurityGroupName(FormatKeyword):
    id = "E1153"
    shortdesc = "Validate security group name"
    description = "Security group names have to valid pattern"
    tags = []
    source_url = "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/format_keyword.md#AWS::EC2::SecurityGroup.GroupName"

    def __init__(self):
        super().__init__(format="AWS::EC2::SecurityGroup.GroupName")

    def format(self, validator: Validator, instance: Any) -> bool:
        if not isinstance(instance, str):
            return True

        if re.match(r"^[a-zA-Z0-9 \._\-:\/()#\,@\[\]+=&;\{\}!\$\*]+$", instance):
            return True

        return False



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_lambda_function
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class FunctionZipfileRuntimeEnum(CfnLintJsonSchema):
    id = "E3677"
    shortdesc = "Validate Lambda using ZipFile requires an allowable runtime"
    description = (
        "Using the ZipFile attribute requires a javascript or "
        "python runtime to be specified"
    )
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=["Resources/AWS::Lambda::Function/Properties"],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_lambda_function,
                filename="zipfile_runtime_enum.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return err.message



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque
from typing import Any

import pytest

from cfnlint.jsonschema._resolvers_cfn import ResolutionResult, Validator
from cfnlint.jsonschema.exceptions import ValidationError
from cfnlint.jsonschema.validators import CfnTemplateValidator


def _resolver(validator: Validator, instance: Any) -> ResolutionResult:
    if instance == "fail":
        yield None, validator, ValidationError("Foo")
        return

    yield "foo", validator, None


@pytest.fixture
def validator():
    validator = CfnTemplateValidator({})
    validator.fn_resolvers = {
        "Ref": _resolver,
    }
    validator.context = validator.context.evolve(functions=["Ref"])
    return validator


@pytest.mark.parametrize(
    "name,instance,expected",
    [
        (
            "Valid with function",
            {"Ref": "fail"},
            [(None, ValidationError("Foo", path=deque(["Ref"])))],
        ),
        ("Valid string", "foo", [("foo", None)]),
        ("Valid with function", {"Ref": "pass"}, [("foo", None)]),
    ],
)
def test_validator_resolver(name, instance, expected, validator):
    results = list(validator.resolve_value(instance))

    if len(results) > len(expected):
        for result, expected_result in zip(results, expected):
            assert result[0] == expected_result[0]
            assert result[2] == expected_result[1]
    else:
        for expected_result, result in zip(expected, results):
            assert result[0] == expected_result[0]
            assert result[2] == expected_result[1]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

import cfnlint.data.schemas.other.resources
import cfnlint.helpers
from cfnlint.jsonschema import ValidationResult, Validator
from cfnlint.jsonschema._keywords import additionalProperties, properties
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class Metadata(CfnLintJsonSchema):
    """Check Base Resource Configuration"""

    id = "E3028"
    shortdesc = "Validate the metadata section of a resource"
    description = (
        "The metadata section can be unstructured but we do "
        "validate the items we can"
    )
    source_url = "https://github.com/aws-cloudformation/cfn-lint"
    tags = ["resources"]

    def __init__(self):
        super().__init__(
            keywords=["Resources/*/Metadata"],
            schema_details=SchemaDetails(
                cfnlint.data.schemas.other.resources, "metadata.json"
            ),
            all_matches=True,
        )
        self.validators["properties"] = self._properties
        self.validators["additionalProperties"] = self._additional_properties

    def _properties(self, validator: Validator, pP: Any, instance: Any, schema: Any):
        # We have to rework pattern properties
        # to re-add the keyword or we will have an
        # infinite loop
        validator = validator.evolve(
            function_filter=validator.function_filter.evolve(
                add_cfn_lint_keyword=True,
            )
        )

        yield from properties(validator, pP, instance, schema)

    def _additional_properties(
        self, validator: Validator, pP: Any, instance: Any, schema: Any
    ):
        # We have to rework pattern properties
        # to re-add the keyword or we will have an
        # infinite loop
        validator = validator.evolve(
            function_filter=validator.function_filter.evolve(
                add_cfn_lint_keyword=True,
            )
        )

        yield from additionalProperties(validator, pP, instance, schema)

    def validate(
        self, validator: Validator, keywords: Any, instance: Any, schema: Any
    ) -> ValidationResult:
        validator = self.extend_validator(
            validator=validator,
            schema=self._schema,
            context=validator.context.evolve(
                functions=cfnlint.helpers.FUNCTIONS,
                strict_types=False,
            ),
        )

        yield from self._iter_errors(validator, instance)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

from cfnlint.helpers import create_rules
from cfnlint.rules import CloudFormationLintRule


class TestCreateRules(BaseTestCase):
    """Test creating rules from a module."""

    def testBase(self):
        from cfnlint.rules.jsonschema import JsonSchema

        rules = create_rules(JsonSchema)
        self.assertTrue(all(isinstance(r, CloudFormationLintRule) for r in rules))
        self.assertTrue("E1001" in (r.id for r in rules))



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import fnmatch
import json
import os
import pathlib
from typing import Any, List
from unittest import TestCase

import pytest
import regex as re

import cfnlint
from cfnlint.helpers import REGIONS, ensure_list, load_plugins, load_resource
from cfnlint.jsonschema import StandardValidator, ValidationError
from cfnlint.schema.resolver import RefResolutionError, RefResolver


@pytest.mark.data
class TestSchemaFiles(TestCase):
    """Test schema files"""

    _found_keywords: List[str] = [
        "*",
        "Conditions",
        "Description",
        "Mappings",
        "Metadata",
        "Metadata/AWS::CloudFormation::Interface",
        "Metadata/cfn-lint",
        "Outputs",
        "Outputs/*",
        "Outputs/*/Condition",
        "Outputs/*/Export/Name",
        "Outputs/*/Value",
        "Parameters",
        "Parameters/*",
        "Resources",
        "Resources/*",
        "Resources/*/Condition",
        "Resources/*/DeletionPolicy",
        "Resources/*/DependsOn",
        "Resources/*/DependsOn/*",
        "Resources/*/Metadata",
        "Resources/*/Metadata/AWS::CloudFormation::Init",
        "Resources/*/Type",
        "Resources/*/UpdatePolicy",
        "Resources/*/UpdateReplacePolicy",
        "Transform",
    ]

    def setUp(self) -> None:
        schema_path = os.path.join(os.path.dirname(cfnlint.__file__), "data", "schemas")
        self.paths = {
            "extensions": os.path.join(schema_path, "extensions"),
            "providers": os.path.join(schema_path, "providers"),
            "other": os.path.join(schema_path, "other"),
            "fixtures": os.path.join(
                os.path.dirname(__file__),
                "..",
                "fixtures",
                "schemas",
                "providers",
            ),
        }
        filename = os.path.join(
            self.paths["fixtures"], "..", "json_schema", "draft7.json"
        )
        with open(filename, "r") as fh:
            d = json.load(fh)
            self.schema_draft7 = d

        super().setUp()

    def get_files(self, dir):
        for dirpath, _, filenames in os.walk(dir):
            for filename in fnmatch.filter(filenames, "*.json"):
                yield dirpath, filename

    def pattern(self, validator, patrn, instance, schema):
        try:
            re.compile(patrn)
        except Exception:
            yield ValidationError(f"Pattern doesn't compile: {patrn}")

    def validate_basic_schema_details(
        self, schema_resolver: RefResolver, filepath: str
    ):
        """
        Validate that readOnly, writeOnly, etc are valid
        """
        sections = [
            "readOnlyProperties",
            "writeOnlyProperties",
            "conditionalCreateOnlyProperties",
            "nonPublicProperties",
            "nonPublicDefinitions",
            "createOnlyProperties",
            "deprecatedProperties",
            "primaryIdentifier",
        ]
        for section in sections:
            for prop in schema_resolver.referrer.get(section, []):
                try:
                    self.assertIsNotNone(schema_resolver.resolve_cfn_pointer(prop))
                except RefResolutionError:
                    self.fail(f"Can't find prop {prop} for {section} in {filepath}")

    def _build_keywords(
        self, obj: Any, schema_resolver: RefResolver, refs: list[str] | None = None
    ):
        if refs is None:
            refs = []
        if not isinstance(obj, dict):
            yield []
            return

        if "type" in obj:
            if "object" in ensure_list(obj["type"]):
                if "properties" in obj:
                    for k, v in obj["properties"].items():
                        for item in self._build_keywords(v, schema_resolver, refs):
                            yield [k] + item
            if "array" in ensure_list(obj["type"]):
                if "items" in obj:
                    for item in self._build_keywords(
                        obj["items"], schema_resolver, refs
                    ):
                        yield ["*"] + item

        if "$ref" in obj:
            ref = obj["$ref"]
            if ref in refs:
                yield []
                return
            _, resolved_schema = schema_resolver.resolve(ref)
            for item in self._build_keywords(
                resolved_schema, schema_resolver, refs + [ref]
            ):
                yield item

        yield []

    def build_keywords(self, schema_resolver):
        self._found_keywords.append(
            "/".join(["Resources", schema_resolver.referrer["typeName"], "Properties"])
        )
        for k, v in schema_resolver.referrer.get("properties").items():
            for item in self._build_keywords(v, schema_resolver):
                self._found_keywords.append(
                    "/".join(
                        [
                            "Resources",
                            schema_resolver.referrer["typeName"],
                            "Properties",
                            k,
                        ]
                        + item
                    )
                )

    def test_data_module_specs(self):
        """Test data file formats"""

        draft7_schema = load_resource(
            "cfnlint.data.schemas.other.draft7", "schema.json"
        )
        store = {"http://json-schema.org/draft-07/schema": draft7_schema}
        dir = self.paths["fixtures"]
        for dirpath, filename in self.get_files(dir):
            with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                store[filename] = json.load(fh)

        resolver = RefResolver.from_schema(
            store["provider.definition.schema.v1.json"], store=store
        )

        validator = (
            StandardValidator({})
            .extend(
                validators={
                    "cfnLint": self.cfn_lint,
                    "pattern": self.pattern,
                },
            )(schema=store["provider.definition.schema.v1.json"])
            .evolve(resolver=resolver)
        )

        for region in REGIONS:
            dir = os.path.join(
                self.paths["providers"],
                region.replace("-", "_"),
            )

            for dirpath, filename in self.get_files(dir):
                with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                    d = json.load(fh)
                    # not allowed but true with this resource
                    if filename == "aws-cloudformation-customresource.json":
                        d["additionalProperties"] = False
                    if filename == "module.json":
                        continue
                    errs = list(validator.iter_errors(d))
                    self.assertListEqual(
                        errs, [], f"Error with {dirpath}/{filename}: {errs}"
                    )
                    schema_resolver = RefResolver(d)
                    self.validate_basic_schema_details(
                        schema_resolver, f"{dirpath}/{filename}"
                    )
                    self.build_keywords(schema_resolver)

    def cfn_lint(self, validator, _, keywords, schema):
        keywords = ensure_list(keywords)
        self._found_keywords.extend(keywords)

    def test_other_specs(self):
        """Test data file formats"""

        draft7_schema = load_resource(
            "cfnlint.data.schemas.other.draft7", "schema.json"
        )
        store = {"http://json-schema.org/draft-07/schema": draft7_schema}
        dir = self.paths["fixtures"]
        for dirpath, filename in self.get_files(dir):
            with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                store[filename] = json.load(fh)

        resolver = RefResolver.from_schema(
            store["http://json-schema.org/draft-07/schema"], store=store
        )

        validator = (
            StandardValidator({})
            .extend(
                validators={
                    "cfnLint": self.cfn_lint,
                    "pattern": self.pattern,
                },
            )(schema=store["http://json-schema.org/draft-07/schema"])
            .evolve(resolver=resolver)
        )

        for dir_name in ["extensions", "other"]:
            dir = self.paths[dir_name]

            for dirpath, filename in self.get_files(dir):
                with open(os.path.join(dirpath, filename), "r", encoding="utf8") as fh:
                    d = json.load(fh)
                    errs = list(validator.iter_errors(d))
                    self.assertListEqual(
                        errs, [], f"Error with {dirpath}/{filename}: {errs}"
                    )
                    schema_resolver = RefResolver(d)
                    self.validate_basic_schema_details(
                        schema_resolver, f"{dirpath}/{filename}"
                    )

    def test_x_keywords(self):
        root_dir = pathlib.Path(__file__).parent.parent.parent / "src/cfnlint/rules"
        rules = load_plugins(
            str(root_dir),
            "CfnLintKeyword",
            "cfnlint.rules.jsonschema.CfnLintKeyword",
        )
        rules.extend(
            load_plugins(
                str(root_dir),
                "CfnLintJsonSchema",
                "cfnlint.rules.jsonschema.CfnLintJsonSchema",
            )
        )

        for rule in rules:
            for keyword in rule.keywords:
                self.assertIn(keyword, self._found_keywords, f"{keyword} not found")



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.context import Path
from cfnlint.rules.parameters.ValuePattern import ValuePattern as ParameterPattern
from cfnlint.rules.resources.properties.Pattern import Pattern


@pytest.fixture(scope="module")
def rule():
    rule = Pattern()
    rule.child_rules["W2031"] = ParameterPattern()
    yield rule


def test_validate(rule, validator):
    assert len(list(rule.pattern(validator, ".*", "foo", {}))) == 0
    assert len(list(rule.pattern(validator, "foo", "bar", {}))) == 1

    evolved = validator.evolve(
        context=validator.context.evolve(
            path=Path(path=deque(["Fn::Sub"])),
        )
    )
    errs = list(rule.pattern(evolved, "bar", "bar", {}))
    assert len(errs) == 0

    evolved = validator.evolve(
        context=validator.context.evolve(
            path=Path(
                path=deque(["Ref"]),
                value_path=deque(["Parameters", "MyParameter", "Default"]),
            ),
        )
    )
    errs = list(rule.pattern(evolved, "foo", "bar", {}))
    assert len(errs) == 1
    assert errs[0].rule.id == ParameterPattern.id

    rule.child_rules["W2031"] = None
    errs = list(rule.pattern(evolved, "foo", "bar", {}))
    assert len(errs) == 0


def test_pattern_exceptions(rule, validator):
    rule.configure({"exceptions": ["AWS::"]})

    assert len(list(rule.pattern(validator, "foo", "Another AWS::Instance", {}))) == 1
    assert len(list(rule.pattern(validator, "foo", "AWS::Dummy::Resource", {}))) == 0



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.lmbd.FunctionEnvironmentKeys import FunctionEnvironmentKeys


@pytest.fixture(scope="module")
def rule():
    rule = FunctionEnvironmentKeys()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {"Foo": "Bar"},
            [],
        ),
        (
            {"AWS_REGION": "Bar"},
            [
                ValidationError(
                    (
                        "'AWS_REGION' is a reserved variable name, one of "
                        "['_HANDLER', '_X_AMZN_TRACE_ID', 'AWS_DEFAULT_REGION', "
                        "'AWS_REGION', 'AWS_EXECUTION_ENV', "
                        "'AWS_LAMBDA_FUNCTION_NAME', "
                        "'AWS_LAMBDA_FUNCTION_MEMORY_SIZE', "
                        "'AWS_LAMBDA_FUNCTION_VERSION', "
                        "'AWS_LAMBDA_INITIALIZATION_TYPE', "
                        "'AWS_LAMBDA_LOG_GROUP_NAME', "
                        "'AWS_LAMBDA_LOG_STREAM_NAME', "
                        "'AWS_ACCESS_KEY', 'AWS_ACCESS_KEY_ID', "
                        "'AWS_SECRET_ACCESS_KEY', "
                        "'AWS_SESSION_TOKEN', 'AWS_LAMBDA_RUNTIME_API', "
                        "'LAMBDA_TASK_ROOT', 'LAMBDA_RUNTIME_DIR']"
                    ),
                    schema_path=deque(["propertyNames", "not"]),
                    path=deque(["AWS_REGION"]),
                    rule=FunctionEnvironmentKeys(),
                    validator="not",
                )
            ],
        ),
        (
            {
                "Foo": "Bar",
                "AWS_REGION": "Bar",
                "Bar": "Foo",
                "AWS_ACCESS_KEY": "Foo",
            },
            [
                ValidationError(
                    (
                        "'AWS_REGION' is a reserved variable name, one of "
                        "['_HANDLER', '_X_AMZN_TRACE_ID', 'AWS_DEFAULT_REGION', "
                        "'AWS_REGION', 'AWS_EXECUTION_ENV', "
                        "'AWS_LAMBDA_FUNCTION_NAME', "
                        "'AWS_LAMBDA_FUNCTION_MEMORY_SIZE', "
                        "'AWS_LAMBDA_FUNCTION_VERSION', "
                        "'AWS_LAMBDA_INITIALIZATION_TYPE', "
                        "'AWS_LAMBDA_LOG_GROUP_NAME', "
                        "'AWS_LAMBDA_LOG_STREAM_NAME', "
                        "'AWS_ACCESS_KEY', 'AWS_ACCESS_KEY_ID', "
                        "'AWS_SECRET_ACCESS_KEY', "
                        "'AWS_SESSION_TOKEN', 'AWS_LAMBDA_RUNTIME_API', "
                        "'LAMBDA_TASK_ROOT', 'LAMBDA_RUNTIME_DIR']"
                    ),
                    schema_path=deque(["propertyNames", "not"]),
                    path=deque(["AWS_REGION"]),
                    rule=FunctionEnvironmentKeys(),
                    validator="not",
                ),
                ValidationError(
                    (
                        "'AWS_ACCESS_KEY' is a reserved variable name, one of "
                        "['_HANDLER', '_X_AMZN_TRACE_ID', 'AWS_DEFAULT_REGION', "
                        "'AWS_REGION', 'AWS_EXECUTION_ENV', "
                        "'AWS_LAMBDA_FUNCTION_NAME', "
                        "'AWS_LAMBDA_FUNCTION_MEMORY_SIZE', "
                        "'AWS_LAMBDA_FUNCTION_VERSION', "
                        "'AWS_LAMBDA_INITIALIZATION_TYPE', "
                        "'AWS_LAMBDA_LOG_GROUP_NAME', "
                        "'AWS_LAMBDA_LOG_STREAM_NAME', "
                        "'AWS_ACCESS_KEY', 'AWS_ACCESS_KEY_ID', "
                        "'AWS_SECRET_ACCESS_KEY', "
                        "'AWS_SESSION_TOKEN', 'AWS_LAMBDA_RUNTIME_API', "
                        "'LAMBDA_TASK_ROOT', 'LAMBDA_RUNTIME_DIR']"
                    ),
                    schema_path=deque(["propertyNames", "not"]),
                    path=deque(["AWS_ACCESS_KEY"]),
                    rule=FunctionEnvironmentKeys(),
                    validator="not",
                ),
            ],
        ),
    ],
)
def test_validate(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "LambdaRuntime", instance, {}))
    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from pathlib import Path
from test.integration import BaseCliTestCase


class TestDirectives(BaseCliTestCase):
    """Test Directives"""

    # ruff: noqa: E501
    scenarios = [
        {
            "filename": str(Path("test/fixtures/templates/good/core/directives.yaml")),
            "exit_code": 0,
            "results": [],
        },
        {
            "filename": str(Path("test/fixtures/templates/bad/core/directives.yaml")),
            "exit_code": 2,
            "results": [
                {
                    "Filename": str(
                        Path("test/fixtures/templates/bad/core/directives.yaml")
                    ),
                    "Id": "4d8d1fd7-496f-9420-44df-f053d372c3be",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 18, "LineNumber": 17},
                        "Path": [
                            "Resources",
                            "myBucketFail",
                            "Properties",
                            "BucketName1",
                        ],
                        "Start": {"ColumnNumber": 7, "LineNumber": 17},
                    },
                    "Message": "Additional properties are not allowed ('BucketName1' was unexpected. Did you mean 'BucketName'?)",
                    "ParentId": None,
                    "Rule": {
                        "Description": (
                            "Making sure that resources properties are properly"
                            " configured"
                        ),
                        "Id": "E3002",
                        "ShortDescription": "Resource properties are invalid",
                        "Source": "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#properties",
                    },
                },
                {
                    "Filename": str(
                        Path("test/fixtures/templates/bad/core/directives.yaml")
                    ),
                    "Id": "dd126795-8106-b986-19ed-db769ee48fba",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 13, "LineNumber": 28},
                        "Path": [
                            "Resources",
                            "myBucketFirstAndLastPass",
                            "Properties",
                            "BadKey",
                        ],
                        "Start": {"ColumnNumber": 7, "LineNumber": 28},
                    },
                    "Message": "Additional properties are not allowed ('BadKey' was unexpected)",
                    "ParentId": None,
                    "Rule": {
                        "Description": (
                            "Making sure that resources properties are properly"
                            " configured"
                        ),
                        "Id": "E3002",
                        "ShortDescription": "Resource properties are invalid",
                        "Source": "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#properties",
                    },
                },
                {
                    "Filename": str(
                        Path("test/fixtures/templates/bad/core/directives.yaml")
                    ),
                    "Id": "431970a8-482e-2aca-af3f-5d4f819f0b45",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 16, "LineNumber": 32},
                        "Path": [
                            "Resources",
                            "myBucketFirstAndLastFail",
                            "BadProperty",
                        ],
                        "Start": {"ColumnNumber": 5, "LineNumber": 32},
                    },
                    "Message": "Additional properties are not allowed ('BadProperty' was unexpected)",
                    "ParentId": None,
                    "Rule": {
                        "Description": (
                            "Making sure the basic CloudFormation resources are"
                            " properly configured"
                        ),
                        "Id": "E3001",
                        "ShortDescription": "Basic CloudFormation Resource Check",
                        "Source": ("https://github.com/aws-cloudformation/cfn-lint"),
                    },
                },
                {
                    "Filename": str(
                        Path("test/fixtures/templates/bad/core/directives.yaml")
                    ),
                    "Id": "814e8b07-dcef-f3ac-4285-8032d1fb010e",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 13, "LineNumber": 35},
                        "Path": [
                            "Resources",
                            "myBucketFirstAndLastFail",
                            "Properties",
                            "BadKey",
                        ],
                        "Start": {"ColumnNumber": 7, "LineNumber": 35},
                    },
                    "Message": "Additional properties are not allowed ('BadKey' was unexpected)",
                    "ParentId": None,
                    "Rule": {
                        "Description": (
                            "Making sure that resources properties are properly"
                            " configured"
                        ),
                        "Id": "E3002",
                        "ShortDescription": "Resource properties are invalid",
                        "Source": "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#properties",
                    },
                },
                {
                    "Filename": str(
                        Path("test/fixtures/templates/bad/core/directives.yaml")
                    ),
                    "Id": "2637b826-b8a7-4e03-8b27-bf8e8d4d12ee",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 15, "LineNumber": 37},
                        "Path": [
                            "Resources",
                            "myBucketFirstAndLastFail",
                            "Properties",
                            "VersioningConfiguration",
                            "Status",
                        ],
                        "Start": {"ColumnNumber": 9, "LineNumber": 37},
                    },
                    "Message": "'Enabled1' is not one of ['Enabled', 'Suspended']",
                    "ParentId": None,
                    "Rule": {
                        "Description": (
                            "Check if properties have a valid value in case of an"
                            " enumator"
                        ),
                        "Id": "E3030",
                        "ShortDescription": "Check if properties have a valid value",
                        "Source": "https://github.com/aws-cloudformation/cfn-lint/blob/main/docs/cfn-schema-specification.md#enum",
                    },
                },
            ],
        },
    ]

    def test_templates(self):
        """Test ignoring certain rules"""
        self.run_scenarios()



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

import logging
from typing import Any, Sequence

from cfnlint.context import Context
from cfnlint.jsonschema import V, ValidationError, Validator
from cfnlint.rules import CloudFormationLintRule, RuleMatch
from cfnlint.schema.resolver import RefResolver

LOGGER = logging.getLogger("cfnlint.rules.jsonschema")


class BaseJsonSchema(CloudFormationLintRule):
    """The base JSON schema package"""

    def __init__(self) -> None:
        """Init"""
        super().__init__()
        self.rules: dict[str, Any] = {}
        self.rule_set: dict[str, str] = {}
        self.validators: dict[str, V] = {}

    def _convert_validation_errors_to_matches(
        self,
        path: Sequence[str],
        e: ValidationError,
    ):
        matches = []
        kwargs: dict[Any, Any] = {}
        if e.extra_args:
            kwargs = e.extra_args
        e_path = list(path) + list(e.path)
        if len(e.path) > 0:
            e_path_override = e.path_override
            if e_path_override:
                e_path = list(e_path_override)
            else:
                key = e.path[-1]
                if hasattr(key, "start_mark"):
                    kwargs["location"] = (
                        key.start_mark.line,
                        key.start_mark.column,
                        key.end_mark.line,
                        key.end_mark.column,
                    )
        e_rule = None
        if e.rule:
            e_rule = e.rule
        if not e_rule:
            rs_rule = self.child_rules.get(self.rule_set.get(e.validator, ""), self)
            # if the value is None it means the rule was disabled
            # so we continue
            if rs_rule is None:
                return []

            e_rule = rs_rule

        match = RuleMatch(
            e_path,
            e.message,
            rule=e_rule,
            context=[],
            **kwargs,
        )

        matches.append(match)

        if e.context:
            for err in e.context:
                match.context.extend(
                    self._convert_validation_errors_to_matches(path, err)
                )

        return matches

    def json_schema_validate(
        self, validator: Validator, properties: dict[str, Any], path: Sequence[str]
    ):
        matches = []
        for err in validator.iter_errors(properties):
            matches.extend(self._convert_validation_errors_to_matches(path, err))

        return matches

    @property
    def schema(self) -> Any:
        return {}

    def _clean_error(self, err: ValidationError):
        if err.rule is None:
            if err.validator:
                if err.validator in self.rule_set:
                    err.rule = self.child_rules[self.rule_set[err.validator]]
                elif not err.validator.startswith("fn") and err.validator != "ref":
                    err.rule = self
            else:
                err.rule = self
        for i, c in enumerate(err.context):
            err.context[i] = self._clean_error(c)
        return err

    def _validate(self, validator: Validator, instance: Any):
        for err in validator.iter_errors(instance):
            yield self._clean_error(err)

    # pylint: disable=unused-argument
    def validate(self, validator: Validator, _, instance: Any, schema):
        validator = self.extend_validator(
            validator, self.schema, validator.context.evolve()
        )
        yield from self._validate(validator, instance)

    def _get_validators(self) -> dict[str, V]:
        validators = self.validators.copy()
        for name, rule_id in self.rule_set.items():
            rule = self.child_rules.get(rule_id)
            if rule is not None:
                if hasattr(rule, name) and callable(getattr(rule, name)):
                    validators[name] = getattr(rule, name)

        return validators

    def extend_validator(
        self, validator: Validator, schema: Any, context: Context
    ) -> Validator:
        return validator.extend(validators=self._get_validators())(
            schema=schema
        ).evolve(
            cfn=validator.cfn,
            context=context,
            resolver=RefResolver.from_schema(
                schema,
            ),
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.formatters.base import BaseFormatter


class QuietFormatter(BaseFormatter):
    """Quiet Formatter"""

    def _format(self, match):
        """Format output"""
        formatstr = "{0} {1}:{2}"
        return formatstr.format(match.rule, match.filename, match.linenumber)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from collections import deque

import pytest

from cfnlint.jsonschema import ValidationError
from cfnlint.rules.resources.backup.BackupPlanLifecycleRule import (
    BackupPlanLifecycleRule,
)


@pytest.fixture(scope="module")
def rule():
    rule = BackupPlanLifecycleRule()
    yield rule


@pytest.mark.parametrize(
    "instance,expected",
    [
        (
            {},
            [],
        ),
        (
            {
                "DeleteAfterDays": 100,
                "MoveToColdStorageAfterDays": 10,
            },
            [],
        ),
        (
            {
                "DeleteAfterDays": "foo",
                "MoveToColdStorageAfterDays": 10,
            },
            [],
        ),
        (
            {
                "DeleteAfterDays": 100,
                "MoveToColdStorageAfterDays": "foo",
            },
            [],
        ),
        (
            {
                "DeleteAfterDays": 10,
                "MoveToColdStorageAfterDays": 30,
            },
            [
                ValidationError(
                    (
                        "DeleteAfterDays 10 must be at least 90 "
                        "days after MoveToColdStorageAfterDays 30"
                    ),
                    rule=BackupPlanLifecycleRule(),
                    path=deque(["DeleteAfterDays"]),
                )
            ],
        ),
    ],
)
def test_backup_lifecycle(instance, expected, rule, validator):
    errs = list(rule.validate(validator, "LambdaRuntime", instance, {}))
    assert errs == expected, f"Expected {expected} got {errs}"



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules._rule import CloudFormationLintRule


class TransformError(CloudFormationLintRule):
    """Transform Lint Rule"""

    id = "E0001"
    shortdesc = "Error found when transforming the template"
    description = "Errors found when performing transformation on the template"
    source_url = "https://github.com/aws-cloudformation/cfn-lint"
    tags = ["base", "transform"]



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from datetime import datetime
from typing import Any

from cfnlint.data import AdditionalSpecs
from cfnlint.helpers import load_resource
from cfnlint.jsonschema import ValidationError, ValidationResult, Validator
from cfnlint.rules.jsonschema.CfnLintKeyword import CfnLintKeyword


class DeprecatedRuntimeEol(CfnLintKeyword):

    id = "W2531"
    shortdesc = "Check if EOL Lambda Function Runtimes are used"
    description = (
        "Check if an EOL Lambda Runtime is specified and give a warning if used. "
    )
    source_url = (
        "https://docs.aws.amazon.com/lambda/latest/dg/runtime-support-policy.html"
    )
    tags = ["resources", "lambda", "runtime"]

    def __init__(self):
        """Init"""
        super().__init__(["Resources/AWS::Lambda::Function/Properties/Runtime"])
        self.current_date = datetime.today()
        self.deprecated_runtimes = load_resource(
            AdditionalSpecs, "LmbdRuntimeLifecycle.json"
        )

    def validate(
        self, validator: Validator, v: Any, runtime: Any, schema: dict[str, Any]
    ) -> ValidationResult:
        if not validator.is_type(runtime, "string"):
            return

        runtime_data = self.deprecated_runtimes.get(runtime)
        if not runtime_data:
            return
        if (
            datetime.strptime(runtime_data["deprecated"], "%Y-%m-%d")
            <= self.current_date
            and datetime.strptime(runtime_data["create-block"], "%Y-%m-%d")
            > self.current_date
            and datetime.strptime(runtime_data["update-block"], "%Y-%m-%d")
            > self.current_date
        ):
            yield ValidationError(
                (
                    f"Runtime {runtime!r} was deprecated on "
                    f"{runtime_data['deprecated']!r}. Creation was disabled on "
                    f"{runtime_data['create-block']!r} and update on "
                    f"{runtime_data['update-block']!r}. Please consider "
                    f"updating to {runtime_data['successor']!r}"
                ),
            )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.extensions.aws_ec2_securitygroup
from cfnlint.jsonschema import ValidationError
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class SecurityGroupProtocolsAndPortsExclusive(CfnLintJsonSchema):
    id = "W3687"
    shortdesc = "Validate that ports aren't specified for certain protocols"
    description = (
        "When using a protocol other than icmp, icmpv6, tcp, or udp "
        "the port ranges properties are ignored"
    )
    tags = ["resources"]

    def __init__(self) -> None:
        super().__init__(
            keywords=[
                "Resources/AWS::EC2::SecurityGroup/Properties/SecurityGroupIngress/*",
                "Resources/AWS::EC2::SecurityGroup/Properties/SecurityGroupEgress/*",
                "Resources/AWS::EC2::SecurityGroupEgress/Properties",
                "Resources/AWS::EC2::SecurityGroupIngress/Properties",
            ],
            schema_details=SchemaDetails(
                module=cfnlint.data.schemas.extensions.aws_ec2_securitygroup,
                filename="protocols_and_port_ranges_exclude.json",
            ),
        )

    def message(self, instance: Any, err: ValidationError) -> str:
        return (
            "['FromPort', 'ToPort'] are ignored when using "
            f"'IpProtocol' value {instance.get('IpProtocol')!r}"
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from typing import Any

from cfnlint.jsonschema import ValidationError, Validator
from cfnlint.rules import CloudFormationLintRule


class DynamicReferenceSecretsManagerPath(CloudFormationLintRule):
    id = "E1051"
    shortdesc = (
        "Validate dynamic references to secrets manager are only in resource properties"
    )
    description = (
        "Dynamic references from secrets manager can only be used "
        "in resource properties"
    )
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/dynamic-references.html#dynamic-references-secretsmanager"
    tags = ["functions", "dynamic reference"]

    def validate(self, validator: Validator, s: Any, instance: Any, schema: Any):
        if len(validator.context.path.path) >= 3:
            if (
                validator.context.path.path[0] == "Resources"
                and validator.context.path.path[2] == "Properties"
            ):
                return

        yield ValidationError(
            (
                f"Dynamic reference {instance!r} to secrets manager can only be "
                "used in resource properties"
            ),
            rule=self,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from __future__ import annotations

from typing import Any

import cfnlint.data.schemas.other.outputs
from cfnlint.jsonschema import Validator
from cfnlint.jsonschema._keywords import patternProperties
from cfnlint.jsonschema._keywords_cfn import cfn_type
from cfnlint.rules.jsonschema.CfnLintJsonSchema import CfnLintJsonSchema, SchemaDetails


class Configuration(CfnLintJsonSchema):
    """Check Base Outputs Configuration"""

    id = "E6001"
    shortdesc = "Check the properties of Outputs"
    description = "Validate the property structure for outputs"
    source_url = "https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/outputs-section-structure.html"
    tags = ["outputs"]

    def __init__(self):
        """Init"""
        super().__init__(
            keywords=["Outputs"],
            schema_details=SchemaDetails(
                cfnlint.data.schemas.other.outputs, "configuration.json"
            ),
            all_matches=True,
        )
        self.rule_set = {
            "propertyNames": "E6011",
            "maxLength": "E6011",
            "pattern": "E6004",
            "required": "E6002",
            "type": "E6003",
            "maxProperties": "E6010",
        }
        self.child_rules = dict.fromkeys(list(self.rule_set.values()))
        self.validators = {
            "type": cfn_type,
            "patternProperties": self._pattern_properties,
        }

    def _pattern_properties(
        self, validator: Validator, pP: Any, instance: Any, schema: Any
    ):
        # We have to rework pattern properties
        # to re-add the keyword or we will have an
        # infinite loop
        validator = validator.evolve(
            function_filter=validator.function_filter.evolve(
                add_cfn_lint_keyword=True,
            )
        )

        yield from patternProperties(validator, pP, instance, schema)

    def validate(self, validator: Validator, _: Any, instance: Any, schema: Any):
        cfn_validator = self.extend_validator(
            validator=validator,
            schema=self._schema,
            context=validator.context.evolve(strict_types=False),
        )

        yield from super()._iter_errors(cfn_validator, instance)
